@@ -13,38 +13,37 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/****************************************************************************
- * This module is based on Twm, but has been siginificantly modified
+/* This module is based on Twm, but has been siginificantly modified
  * by Rob Nation
- ****************************************************************************/
-/*****************************************************************************/
-/**       Copyright 1988 by Evans & Sutherland Computer Corporation,        **/
-/**                          Salt Lake City, Utah                           **/
-/**  Portions Copyright 1989 by the Massachusetts Institute of Technology   **/
-/**                        Cambridge, Massachusetts                         **/
-/**                                                                         **/
-/**                           All Rights Reserved                           **/
-/**                                                                         **/
-/**    Permission to use, copy, modify, and distribute this software and    **/
-/**    its documentation  for  any  purpose  and  without  fee is hereby    **/
-/**    granted, provided that the above copyright notice appear  in  all    **/
-/**    copies and that both  that  copyright  notice  and  this  permis-    **/
-/**    sion  notice appear in supporting  documentation,  and  that  the    **/
-/**    names of Evans & Sutherland and M.I.T. not be used in advertising    **/
-/**    in publicity pertaining to distribution of the  software  without    **/
-/**    specific, written prior permission.                                  **/
-/**                                                                         **/
-/**    EVANS & SUTHERLAND AND M.I.T. DISCLAIM ALL WARRANTIES WITH REGARD    **/
-/**    TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES  OF  MERCHANT-    **/
-/**    ABILITY  AND  FITNESS,  IN  NO  EVENT SHALL EVANS & SUTHERLAND OR    **/
-/**    M.I.T. BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL  DAM-    **/
-/**    AGES OR  ANY DAMAGES WHATSOEVER  RESULTING FROM LOSS OF USE, DATA    **/
-/**    OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER    **/
-/**    TORTIOUS ACTION, ARISING OUT OF OR IN  CONNECTION  WITH  THE  USE    **/
-/**    OR PERFORMANCE OF THIS SOFTWARE.                                     **/
-/*****************************************************************************/
+ */
+/*
+ *       Copyright 1988 by Evans & Sutherland Computer Corporation,
+ *                          Salt Lake City, Utah
+ *  Portions Copyright 1989 by the Massachusetts Institute of Technology
+ *                        Cambridge, Massachusetts
+ *
+ *                           All Rights Reserved
+ *
+ *    Permission to use, copy, modify, and distribute this software and
+ *    its documentation  for  any  purpose  and  without  fee is hereby
+ *    granted, provided that the above copyright notice appear  in  all
+ *    copies and that both  that  copyright  notice  and  this  permis-
+ *    sion  notice appear in supporting  documentation,  and  that  the
+ *    names of Evans & Sutherland and M.I.T. not be used in advertising
+ *    in publicity pertaining to distribution of the  software  without
+ *    specific, written prior permission.
+ *
+ *    EVANS & SUTHERLAND AND M.I.T. DISCLAIM ALL WARRANTIES WITH REGARD
+ *    TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES  OF  MERCHANT-
+ *    ABILITY  AND  FITNESS,  IN  NO  EVENT SHALL EVANS & SUTHERLAND OR
+ *    M.I.T. BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL  DAM-
+ *    AGES OR  ANY DAMAGES WHATSOEVER  RESULTING FROM LOSS OF USE, DATA
+ *    OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
+ *    TORTIOUS ACTION, ARISING OUT OF OR IN  CONNECTION  WITH  THE  USE
+ *    OR PERFORMANCE OF THIS SOFTWARE.
+ */
 
-/* ---------------------------- included header files ----------------------- */
+/* ---------------------------- included header files ---------------------- */
 
 #include "config.h"
 
@@ -55,6 +54,8 @@
 #include "libs/FScreen.h"
 #include "libs/Picture.h"
 #include "libs/PictureUtils.h"
+#include "libs/charmap.h"
+#include "libs/wcontext.h"
 #include "fvwm.h"
 #include "externs.h"
 #include "cursor.h"
@@ -84,29 +85,46 @@
 #include "colormaps.h"
 #include "decorations.h"
 
-/* ---------------------------- local definitions --------------------------- */
+/* ---------------------------- local definitions -------------------------- */
 
-/* ---------------------------- local macros -------------------------------- */
+/* ---------------------------- local macros ------------------------------- */
 
-/* ---------------------------- imports ------------------------------------- */
+/* ---------------------------- imports ------------------------------------ */
 
-/* ---------------------------- included code files ------------------------- */
+/* ---------------------------- included code files ------------------------ */
 
-/* ---------------------------- local types --------------------------------- */
+/* ---------------------------- local types -------------------------------- */
 
-/* ---------------------------- forward declarations ------------------------ */
+/* ---------------------------- forward declarations ----------------------- */
 
-/* ---------------------------- local variables ----------------------------- */
+/* ---------------------------- local variables ---------------------------- */
 
-/* ---------------------------- exported variables (globals) ---------------- */
+/* ---------------------------- exported variables (globals) --------------- */
 
 char NoName[] = "Untitled"; /* name if no name in XA_WM_NAME */
 char NoClass[] = "NoClass"; /* Class if no res_class in class hints */
 char NoResource[] = "NoResource"; /* Class if no res_name in class hints */
 
-/* ---------------------------- local functions ----------------------------- */
+/* ---------------------------- local functions ---------------------------- */
 
-/***********************************************************************
+static void delete_client_context(FvwmWindow *fw)
+{
+	FvwmWindow *cw;
+
+	/* We can not simply delete the context.  X might have reused the
+	 * window structure so we would delete the context that was established
+	 * by another FvwmWindow structure in the mean time. */
+	if (XFindContext(
+		    dpy, FW_W(fw), FvwmContext, (caddr_t *)&cw) !=
+	    XCNOENT && cw == fw)
+	{
+		XDeleteContext(dpy, FW_W(fw), FvwmContext);
+	}
+
+	return;
+}
+
+/*
  *
  *  Procedure:
  *      CaptureOneWindow
@@ -114,7 +132,7 @@ char NoResource[] = "NoResource"; /* Class if no res_name in class hints */
  *
  *   Decorates windows at start-up and during recaptures
  *
- ***********************************************************************/
+ */
 
 static void CaptureOneWindow(
 	const exec_context_t *exc, FvwmWindow *fw, Window window,
@@ -122,7 +140,7 @@ static void CaptureOneWindow(
 {
 	Window w;
 	unsigned long data[1];
-	initial_window_options_type win_opts;
+	initial_window_options_t win_opts;
 	evh_args_t ea;
 	exec_context_changes_t ecc;
 	XEvent e;
@@ -313,7 +331,7 @@ static void hide_screen(
 	return;
 }
 
-/***********************************************************************
+/*
  *
  *  Procedure:
  *      MappedNotOverride - checks to see if we should really
@@ -326,10 +344,10 @@ static void hide_screen(
  *  Inputs:
  *      w       - the window to check
  *
- ***********************************************************************/
+ */
 
 static int MappedNotOverride(
-	Window w, initial_window_options_type *win_opts)
+	Window w, initial_window_options_t *win_opts)
 {
 	XWindowAttributes wa;
 	Atom atype;
@@ -427,6 +445,7 @@ static void setup_window_structure(
 		SET_USED_TITLE_DIR_FOR_SHADING(
 			*pfw, USED_TITLE_DIR_FOR_SHADING(savewin));
 		SET_SHADED_DIR(*pfw, SHADED_DIR(savewin));
+		SET_NAME_CHANGED(*pfw,IS_NAME_CHANGED(savewin));
 		SET_PLACED_WB3(*pfw,IS_PLACED_WB3(savewin));
 		SET_PLACED_BY_FVWM(*pfw, IS_PLACED_BY_FVWM(savewin));
 		SET_HAS_EWMH_WM_ICON_HINT(*pfw, HAS_EWMH_WM_ICON_HINT(savewin));
@@ -1181,7 +1200,7 @@ static void destroy_auxiliary_windows(
 	{
 		XDeleteContext(dpy, FW_W_FRAME(fw), FvwmContext);
 		XDeleteContext(dpy, FW_W_PARENT(fw), FvwmContext);
-		XDeleteContext(dpy, FW_W(fw), FvwmContext);
+		delete_client_context(fw);
 		XDestroyWindow(dpy, FW_W_FRAME(fw));
 	}
 	if (HAS_TITLE(fw))
@@ -1402,121 +1421,6 @@ static void setup_layer(FvwmWindow *fw, window_style *pstyle)
 	return;
 }
 
-static Bool setup_window_placement(
-	FvwmWindow *fw, window_style *pstyle, rectangle *attr_g,
-	initial_window_options_type *win_opts)
-{
-	int client_argc = 0;
-	char **client_argv = NULL;
-	XrmValue rm_value;
-	int tmpno1 = -1, tmpno2 = -1, tmpno3 = -1, spargs = 0;
-	/* Used to parse command line of clients for specific desk requests. */
-	/* Todo: check for multiple desks. */
-	XrmDatabase db = NULL;
-	static XrmOptionDescRec table [] = {
-		/* Want to accept "-workspace N" or -xrm "fvwm*desk:N" as
-		 * options to specify the desktop. I have to include dummy
-		 * options that are meaningless since Xrm seems to allow -w to
-		 * match -workspace if there would be no ambiguity. */
-		{"-workspacf", "*junk", XrmoptionSepArg, (caddr_t) NULL},
-		{"-workspace", "*desk", XrmoptionSepArg, (caddr_t) NULL},
-		{"-xrn", NULL, XrmoptionResArg, (caddr_t) NULL},
-		{"-xrm", NULL, XrmoptionResArg, (caddr_t) NULL},
-	};
-	Bool rc;
-	const exec_context_t *exc;
-	exec_context_changes_t ecc;
-
-	/* Find out if the client requested a specific desk on the command
-	 * line. */
-	/*  RBW - 11/20/1998 - allow a desk of -1 to work.  */
-
-	if (XGetCommand(dpy, FW_W(fw), &client_argv, &client_argc) &&
-	    client_argc > 0 && client_argv != NULL)
-	{
-		/* Get global X resources */
-		MergeXResources(dpy, &db, False);
-
-		/* command line takes precedence over all */
-		MergeCmdLineResources(
-			&db, table, 4, client_argv[0], &client_argc,
-			client_argv, True);
-
-		/* Now parse the database values: */
-		if (GetResourceString(db, "desk", client_argv[0], &rm_value) &&
-		    rm_value.size != 0)
-		{
-			SGET_START_DESK(*pstyle) = atoi(rm_value.addr);
-			/*  RBW - 11/20/1998  */
-			if (SGET_START_DESK(*pstyle) > -1)
-			{
-				SSET_START_DESK(
-					*pstyle, SGET_START_DESK(*pstyle) + 1);
-			}
-			pstyle->flags.use_start_on_desk = 1;
-		}
-		if (GetResourceString(
-			    db, "fvwmscreen", client_argv[0], &rm_value) &&
-		    rm_value.size != 0)
-		{
-			SSET_START_SCREEN(
-				*pstyle, FScreenGetScreenArgument(
-					rm_value.addr, 'c'));
-			pstyle->flags.use_start_on_screen = 1;
-		}
-		if (GetResourceString(db, "page", client_argv[0], &rm_value) &&
-		    rm_value.size != 0)
-		{
-			spargs = sscanf(
-				rm_value.addr, "%d %d %d", &tmpno1, &tmpno2,
-				&tmpno3);
-			switch (spargs)
-			{
-			case 1:
-				pstyle->flags.use_start_on_desk = 1;
-				SSET_START_DESK(*pstyle, (tmpno1 > -1) ?
-						tmpno1 + 1 : tmpno1);
-				break;
-			case 2:
-				pstyle->flags.use_start_on_desk = 1;
-				SSET_START_PAGE_X(*pstyle, (tmpno1 > -1) ?
-						  tmpno1 + 1 : tmpno1);
-				SSET_START_PAGE_Y(*pstyle, (tmpno2 > -1) ?
-						  tmpno2 + 1 : tmpno2);
-				break;
-			case 3:
-				pstyle->flags.use_start_on_desk = 1;
-				SSET_START_DESK(*pstyle, (tmpno1 > -1) ?
-						tmpno1 + 1 : tmpno1);
-				SSET_START_PAGE_X(*pstyle, (tmpno2 > -1) ?
-						  tmpno2 + 1 : tmpno2);
-				SSET_START_PAGE_Y(*pstyle, (tmpno3 > -1) ?
-						  tmpno3 + 1 : tmpno3);
-				break;
-			default:
-				break;
-			}
-		}
-
-		XFreeStringList(client_argv);
-		XrmDestroyDatabase(db);
-	}
-	if (pstyle->flags.do_start_iconic)
-	{
-		win_opts->initial_state = IconicState;
-	}
-	ecc.type = EXCT_NULL;
-	ecc.w.fw = fw;
-	exc = exc_create_context(&ecc, ECC_TYPE | ECC_FW);
-	rc = PlaceWindow(
-		exc, &pstyle->flags, attr_g, SGET_START_DESK(*pstyle),
-		SGET_START_PAGE_X(*pstyle), SGET_START_PAGE_Y(*pstyle),
-		SGET_START_SCREEN(*pstyle), PLACE_INITIAL, win_opts);
-	exc_destroy_context(exc);
-
-	return rc;
-}
-
 static void destroy_mini_icon(FvwmWindow *fw)
 {
 	if (fw->mini_icon)
@@ -1549,7 +1453,7 @@ static void setup_key_and_button_grabs(FvwmWindow *fw)
 	return;
 }
 
-/* ---------------------------- interface functions ------------------------- */
+/* ---------------------------- interface functions ------------------------ */
 
 void setup_visible_name(FvwmWindow *fw, Bool is_icon)
 {
@@ -1709,9 +1613,8 @@ void setup_window_font(
 void setup_icon_font(
 	FvwmWindow *fw, window_style *pstyle, Bool do_destroy)
 {
-	int height;
+	int height = 0;
 
-	height = (IS_ICON_FONT_LOADED(fw)) ? fw->icon_font->height : 0;
 	if (IS_ICON_SUPPRESSED(fw) || HAS_NO_ICON_TITLE(fw))
 	{
 		if (IS_ICON_FONT_LOADED(fw))
@@ -1747,6 +1650,7 @@ void setup_icon_font(
 		SET_ICON_FONT_LOADED(fw, 1);
 	}
 	/* adjust y position of existing icons */
+	height = (IS_ICON_FONT_LOADED(fw)) ? fw->icon_font->height : 0;
 	if (height)
 	{
 		resize_icon_title_height(fw, height - fw->icon_font->height);
@@ -1763,7 +1667,7 @@ void setup_style_and_decor(
 {
 	/* first copy the static styles into the window struct */
 	memcpy(&(FW_COMMON_FLAGS(fw)), &(SCF(*pstyle)),
-	       sizeof(common_flags_type));
+	       sizeof(common_flags_t));
 	fw->wShaped = None;
 	if (FShapesSupported)
 	{
@@ -1823,7 +1727,7 @@ void setup_style_and_decor(
 	{
 		if (fw->wShaped)
 		{
-			set_window_border_size(fw, 0);
+			set_window_border_size(fw, fw->unshaped_boundary_width);
 			SET_HAS_NO_BORDER(fw, 1);
 			SET_HAS_HANDLES(fw, 0);
 		}
@@ -1851,6 +1755,12 @@ void setup_style_and_decor(
 		GNOME_GetStyle(fw, pstyle);
 	}
 
+	/* ConfigureNotify motion method */
+	if (SCR_MOTION_METHOD(&pstyle->flag_mask))
+	{
+		CR_MOTION_METHOD(fw) = SCR_MOTION_METHOD(&pstyle->flags);
+	}
+
 	return;
 }
 
@@ -2043,6 +1953,121 @@ void setup_focus_policy(FvwmWindow *fw)
 	return;
 }
 
+Bool setup_window_placement(
+	FvwmWindow *fw, window_style *pstyle, rectangle *attr_g,
+	initial_window_options_t *win_opts)
+{
+	int client_argc = 0;
+	char **client_argv = NULL;
+	XrmValue rm_value;
+	int tmpno1 = -1, tmpno2 = -1, tmpno3 = -1, spargs = 0;
+	/* Used to parse command line of clients for specific desk requests. */
+	/* Todo: check for multiple desks. */
+	XrmDatabase db = NULL;
+	static XrmOptionDescRec table [] = {
+		/* Want to accept "-workspace N" or -xrm "fvwm*desk:N" as
+		 * options to specify the desktop. I have to include dummy
+		 * options that are meaningless since Xrm seems to allow -w to
+		 * match -workspace if there would be no ambiguity. */
+		{"-workspacf", "*junk", XrmoptionSepArg, (caddr_t) NULL},
+		{"-workspace", "*desk", XrmoptionSepArg, (caddr_t) NULL},
+		{"-xrn", NULL, XrmoptionResArg, (caddr_t) NULL},
+		{"-xrm", NULL, XrmoptionResArg, (caddr_t) NULL},
+	};
+	Bool rc;
+	const exec_context_t *exc;
+	exec_context_changes_t ecc;
+
+	/* Find out if the client requested a specific desk on the command
+	 * line. */
+	/*  RBW - 11/20/1998 - allow a desk of -1 to work.  */
+
+	if (XGetCommand(dpy, FW_W(fw), &client_argv, &client_argc) &&
+	    client_argc > 0 && client_argv != NULL)
+	{
+		/* Get global X resources */
+		MergeXResources(dpy, &db, False);
+
+		/* command line takes precedence over all */
+		MergeCmdLineResources(
+			&db, table, 4, client_argv[0], &client_argc,
+			client_argv, True);
+
+		/* Now parse the database values: */
+		if (GetResourceString(db, "desk", client_argv[0], &rm_value) &&
+		    rm_value.size != 0)
+		{
+			SGET_START_DESK(*pstyle) = atoi(rm_value.addr);
+			/*  RBW - 11/20/1998  */
+			if (SGET_START_DESK(*pstyle) > -1)
+			{
+				SSET_START_DESK(
+					*pstyle, SGET_START_DESK(*pstyle) + 1);
+			}
+			pstyle->flags.use_start_on_desk = 1;
+		}
+		if (GetResourceString(
+			    db, "fvwmscreen", client_argv[0], &rm_value) &&
+		    rm_value.size != 0)
+		{
+			SSET_START_SCREEN(
+				*pstyle, FScreenGetScreenArgument(
+					rm_value.addr, 'c'));
+			pstyle->flags.use_start_on_screen = 1;
+		}
+		if (GetResourceString(db, "page", client_argv[0], &rm_value) &&
+		    rm_value.size != 0)
+		{
+			spargs = sscanf(
+				rm_value.addr, "%d %d %d", &tmpno1, &tmpno2,
+				&tmpno3);
+			switch (spargs)
+			{
+			case 1:
+				pstyle->flags.use_start_on_desk = 1;
+				SSET_START_DESK(*pstyle, (tmpno1 > -1) ?
+						tmpno1 + 1 : tmpno1);
+				break;
+			case 2:
+				pstyle->flags.use_start_on_desk = 1;
+				SSET_START_PAGE_X(*pstyle, (tmpno1 > -1) ?
+						  tmpno1 + 1 : tmpno1);
+				SSET_START_PAGE_Y(*pstyle, (tmpno2 > -1) ?
+						  tmpno2 + 1 : tmpno2);
+				break;
+			case 3:
+				pstyle->flags.use_start_on_desk = 1;
+				SSET_START_DESK(*pstyle, (tmpno1 > -1) ?
+						tmpno1 + 1 : tmpno1);
+				SSET_START_PAGE_X(*pstyle, (tmpno2 > -1) ?
+						  tmpno2 + 1 : tmpno2);
+				SSET_START_PAGE_Y(*pstyle, (tmpno3 > -1) ?
+						  tmpno3 + 1 : tmpno3);
+				break;
+			default:
+				break;
+			}
+		}
+
+		XFreeStringList(client_argv);
+		XrmDestroyDatabase(db);
+	}
+	if (pstyle->flags.do_start_iconic)
+	{
+		win_opts->initial_state = IconicState;
+	}
+	ecc.type = EXCT_NULL;
+	ecc.w.fw = fw;
+	exc = exc_create_context(&ecc, ECC_TYPE | ECC_FW);
+	rc = PlaceWindow(
+		exc, &pstyle->flags, attr_g, SGET_START_DESK(*pstyle),
+		SGET_START_PAGE_X(*pstyle), SGET_START_PAGE_Y(*pstyle),
+		SGET_START_SCREEN(*pstyle), PLACE_INITIAL, win_opts);
+	exc_destroy_context(exc);
+
+	return rc;
+}
+
 Bool validate_transientfor(FvwmWindow *fw)
 {
 	XWindowAttributes wa;
@@ -2050,7 +2075,7 @@ Bool validate_transientfor(FvwmWindow *fw)
 	Window w;
 
 	w = FW_W_TRANSIENTFOR(fw);
-	if (w == None || w == FW_W(fw))
+	if (w == None || w == FW_W(fw) || w == IS_EWMH_DESKTOP(w))
 	{
 		FW_W_TRANSIENTFOR(fw) = Scr.Root;
 		return False;
@@ -2090,15 +2115,15 @@ Bool setup_transientfor(FvwmWindow *fw)
 	return rc;
 }
 
-/***********************************************************************
+/*
  *
  *  Procedure:
  *      AddWindow - add a new window to the fvwm list
  *
- ***********************************************************************/
+ */
 FvwmWindow *AddWindow(
 	const exec_context_t *exc, FvwmWindow *ReuseWin,
-	initial_window_options_type * win_opts)
+	initial_window_options_t * win_opts)
 {
 	/* new fvwm window structure */
 	register FvwmWindow *fw;
@@ -2136,6 +2161,10 @@ FvwmWindow *AddWindow(
 		    dpy, w, &JunkRoot, &JunkX, &JunkY, &JunkWidth, &JunkHeight,
 		    &JunkBW,  &JunkDepth) == 0)
 	{
+		if (Scr.bo.DisplayNewWindowNames)
+		{
+			fvwm_msg(INFO, "AddWindow", "new window disappeared");
+		}
 		free(fw);
 		MyXUngrabServer(dpy);
 		return NULL;
@@ -2144,6 +2173,31 @@ FvwmWindow *AddWindow(
 	/****** window name ******/
 	setup_window_name(fw);
 	setup_class_and_resource(fw);
+
+	/****** style setup ******/
+	/* get merged styles */
+	lookup_style(fw, &style);
+	sflags = SGET_FLAGS_POINTER(style);
+	if (SIS_UNMANAGED(sflags))
+	{
+		if (Scr.bo.DisplayNewWindowNames)
+		{
+			fvwm_msg(
+				INFO, "AddWindow", "new window is unmanaged:\n"
+				"  name:      %s\n"
+				"  icon name: (unknown)\n"
+				"  resource:  %s\n"
+				"  class:     %s",
+				fw->name.name, fw->class.res_name,
+				fw->class.res_class);
+		}
+		free_window_names(fw, True, True);
+		free(fw);
+		MyXUngrabServer(dpy);
+		return AW_UNMANAGED;
+	}
+
+	/****** window attributes and hints ******/
 	setup_window_attr(fw, &wattr);
 	setup_wm_hints(fw);
 
@@ -2151,9 +2205,6 @@ FvwmWindow *AddWindow(
 	/* If the window is in the NoTitle list, or is a transient, dont
 	 * decorate it.  If its a transient, and DecorateTransients was
 	 * specified, decorate anyway. */
-	/* get merged styles */
-	lookup_style(fw, &style);
-	sflags = SGET_FLAGS_POINTER(style);
 	setup_transientfor(fw);
 	if (win_opts->flags.is_menu)
 	{
@@ -2169,6 +2220,18 @@ FvwmWindow *AddWindow(
 	/***** visible window name ****/
 	setup_visible_name(fw, False);
 	EWMH_SetVisibleName(fw, False);
+	if (Scr.bo.DisplayNewWindowNames)
+	{
+		fvwm_msg(
+			INFO, "AddWindow", "new window:\n"
+			"  name:      %s\n"
+			"  icon name: %s\n"
+			"  resource:  %s\n"
+			"  class:     %s",
+			fw->name.name, (fw->icon_name.name == NULL) ?
+			"(unknown)" : fw->icon_name.name, fw->class.res_name,
+			fw->class.res_class);
+	}
 
 	/****** state setup ******/
 	setup_icon_boxes(fw, &style);
@@ -2295,7 +2358,6 @@ FvwmWindow *AddWindow(
 		fw->frame_g.height = wattr.height + b.total_size.height;
 		gravity_constrain_size(
 			fw->hints.win_gravity, fw, &fw->frame_g, 0);
-
 		update_absolute_geometry(fw);
 	}
 
@@ -2397,7 +2459,7 @@ FvwmWindow *AddWindow(
 		int stick_desk;
 
 		stick_page = is_window_sticky_across_pages(fw);
-		stick_desk = is_window_sticky_across_pages(fw);
+		stick_desk = is_window_sticky_across_desks(fw);
 		if ((stick_page &&
 		     !IsRectangleOnThisPage(&fw->frame_g, Scr.CurrentDesk)) ||
 		    (stick_desk && fw->Desk != Scr.CurrentDesk))
@@ -2508,7 +2570,7 @@ FvwmWindow *AddWindow(
 }
 
 
-/***********************************************************************
+/*
  *
  *  Procedure:
  *      FetchWMProtocols - finds out which protocols the window supports
@@ -2516,7 +2578,7 @@ FvwmWindow *AddWindow(
  *  Inputs:
  *      tmp - the fvwm window structure to use
  *
- ***********************************************************************/
+ */
 void FetchWmProtocols(FvwmWindow *tmp)
 {
 	Atom *protocols = NULL, *ap;
@@ -2584,7 +2646,7 @@ void FetchWmProtocols(FvwmWindow *tmp)
 
 
 
-/***********************************************************************
+/*
  *
  *  Procedure:
  *      GetWindowSizeHints - gets application supplied size info
@@ -2592,7 +2654,7 @@ void FetchWmProtocols(FvwmWindow *tmp)
  *  Inputs:
  *      tmp - the fvwm window structure to use
  *
- ***********************************************************************/
+ */
 
 void GetWindowSizeHints(FvwmWindow *tmp)
 {
@@ -2771,6 +2833,27 @@ void GetWindowSizeHints(FvwmWindow *tmp)
 		tmp->hints.win_gravity = NorthWestGravity;
 	}
 
+	if ((tmp->hints.flags & PMaxSize) &&
+	    ((tmp->hints.flags & PMinSize) || (tmp->hints.flags & PBaseSize)))
+	{
+		if (tmp->hints.max_width < tmp->hints.base_width)
+		{
+			tmp->hints.max_width = DEFAULT_MAX_MAX_WINDOW_WIDTH;
+			if (!*broken_cause)
+			{
+				broken_cause = "max_width";
+			}
+		}
+		if (tmp->hints.max_height < tmp->hints.base_height)
+		{
+			tmp->hints.max_height = DEFAULT_MAX_MAX_WINDOW_HEIGHT;
+			if (!*broken_cause)
+			{
+				broken_cause = "max_height";
+			}
+		}
+	}
+
 	if (tmp->hints.flags & PAspect)
 	{
 		/*
@@ -2815,16 +2898,17 @@ void GetWindowSizeHints(FvwmWindow *tmp)
 			tmp->hints.flags &= ~PAspect;
 			fvwm_msg(
 				WARN, "GetWindowSizeHints",
-				"The applicaton window \"%s\"\n"
-				"(window id %#lx)"
-				" has broken aspect ratio: %d/%d - %d/%d\n"
-				"Fvwm is ignoring the aspect ratio.  "
-				"If you are having a problem\n"
-				"with the application, send this message to"
-				" the application owner.\n"
+				"The applicaton window (window id %#lx)\n"
+				"  \"%s\" has broken aspect ratio: "
+				"%d/%d - %d/%d\n"
+				"    fvwm is ignoring this aspect ratio.  "
+				"If you are having a problem with the\n"
+				"    application, send a bug report with this "
+				"message included to the\n"
+				"    application owner.  "
 				"There is no need to notify "
 				"fvwm-workers@fvwm.org.\n",
-				tmp->name.name, FW_W(tmp), minAspectX,
+				FW_W(tmp), tmp->name.name, minAspectX,
 				minAspectY, maxAspectX, maxAspectY);
 		}
 		else
@@ -2867,42 +2951,43 @@ void GetWindowSizeHints(FvwmWindow *tmp)
 
 	if (*broken_cause != 0)
 	{
-		fvwm_msg(WARN, "GetWindowSizeHints",
-			 "The application window \"%s\"\n(window id %#lx) "
-			 "has broken size hints (%s).\n"
-			 "Fvwm is ignoring those hints.  "
-			 "If you are having a problem\n"
-			 "with the application, send this message to"
-			 " the application owner.\n"
-			 "There is no need to notify fvwm-workers@fvwm.org.\n"
-			 "hint override = %d "
-			 "flags = %lx\n"
-			 "min_width = %d, min_height = %d, "
-			 "max_width = %d, max_height = %d\n"
-			 "width_inc = %d, height_inc = %d\n"
-			 "min_aspect = %d/%d, max_aspect = %d/%d\n"
-			 "base_width = %d, base_height = %d\n"
-			 "win_gravity = %d\n",
-			 tmp->name.name, FW_W(tmp), broken_cause,
-			 HAS_OVERRIDE_SIZE_HINTS(tmp),
-			 orig_hints.flags,
-			 orig_hints.min_width, orig_hints.min_height,
-			 orig_hints.max_width, orig_hints.max_height,
-			 orig_hints.width_inc, orig_hints.height_inc,
-			 orig_hints.min_aspect.x, orig_hints.min_aspect.y,
-			 orig_hints.max_aspect.x, orig_hints.max_aspect.y,
-			 orig_hints.base_width, orig_hints.base_height,
-			 orig_hints.win_gravity);
+		fvwm_msg(
+			WARN, "GetWindowSizeHints",
+			"The application window (id %#lx)\n"
+			"  \"%s\" has broken size hints (%s).\n"
+			"    fvwm is ignoring those hints.  "
+			"If you are having a problem with the\n"
+			"    application, send a bug report with this "
+			"message included to the\n"
+			"    application owner.  "
+			"There is no need to notify fvwm-workers@fvwm.org.\n"
+			"  hint override = %d, flags = %lx\n"
+			"  min_width = %d, min_height = %d, "
+			"max_width = %d, max_height = %d\n"
+			"  width_inc = %d, height_inc = %d\n"
+			"  min_aspect = %d/%d, max_aspect = %d/%d\n"
+			"  base_width = %d, base_height = %d\n"
+			"  win_gravity = %d\n",
+			FW_W(tmp), tmp->name.name, broken_cause,
+			HAS_OVERRIDE_SIZE_HINTS(tmp),
+			orig_hints.flags,
+			orig_hints.min_width, orig_hints.min_height,
+			orig_hints.max_width, orig_hints.max_height,
+			orig_hints.width_inc, orig_hints.height_inc,
+			orig_hints.min_aspect.x, orig_hints.min_aspect.y,
+			orig_hints.max_aspect.x, orig_hints.max_aspect.y,
+			orig_hints.base_width, orig_hints.base_height,
+			orig_hints.win_gravity);
 	}
 
 	return;
 }
 
-/**************************************************************************
+/*
  *
  * Releases dynamically allocated space used to store window/icon names
  *
- **************************************************************************/
+ */
 void free_window_names(FvwmWindow *fw, Bool nukename, Bool nukeicon)
 {
 	if (!fw)
@@ -2910,62 +2995,50 @@ void free_window_names(FvwmWindow *fw, Bool nukename, Bool nukeicon)
 		return;
 	}
 
-	if (nukename && fw->visible_name &&
-	    fw->visible_name != fw->name.name &&
-	    fw->visible_name != NoName)
+	if (nukename)
 	{
-		free(fw->visible_name);
-		fw->visible_name = NULL;
+		if (fw->visible_name && fw->visible_name != fw->name.name &&
+		    fw->visible_name != NoName)
+		{
+			free(fw->visible_name);
+		}
+		fw->visible_name = NoName;
+		if (fw->name.name)
+		{
+			if (fw->icon_name.name == fw->name.name)
+			{
+				fw->icon_name.name = NoName;
+			}
+			if (fw->visible_icon_name == fw->name.name)
+			{
+				fw->visible_icon_name = fw->icon_name.name;
+			}
+			if (fw->name.name != NoName)
+			{
+				FlocaleFreeNameProperty(&(fw->name));
+				fw->visible_name = NULL;
+			}
+		}
 	}
-	if (nukeicon && fw->visible_icon_name &&
-	    fw->visible_icon_name != fw->name.name &&
-	    fw->visible_icon_name != fw->icon_name.name &&
-	    fw->visible_icon_name != NoName)
+	if (nukeicon)
 	{
-		free(fw->visible_icon_name);
-		fw->visible_icon_name = NULL;
-	}
-	if (nukename && fw->name.name)
-	{
-#ifdef CODE_WITH_LEAK_I_THINK
-		if (fw->name.name != fw->icon_name.name &&
-		    fw->name.name != NoName)
+		if (fw->visible_icon_name &&
+		    fw->visible_icon_name != fw->name.name &&
+		    fw->visible_icon_name != fw->icon_name.name &&
+		    fw->visible_icon_name != NoName)
 		{
-			FlocaleFreeNameProperty(&(fw->name));
+			free(fw->visible_icon_name);
 		}
-#else
-		if (fw->icon_name.name == fw->name.name)
+		fw->visible_icon_name = NoName;
+		if (fw->icon_name.name)
 		{
-			fw->icon_name.name = NoName;
+			if ((fw->icon_name.name != fw->name.name) &&
+			    fw->icon_name.name != NoName)
+			{
+				FlocaleFreeNameProperty(&(fw->icon_name));
+				fw->visible_icon_name = NULL;
+			}
 		}
-		if (fw->visible_icon_name == fw->name.name)
-		{
-			fw->visible_icon_name = fw->icon_name.name;
-		}
-		if (fw->name.name != NoName)
-		{
-			FlocaleFreeNameProperty(&(fw->name));
-			fw->visible_name = NULL;
-		}
-#endif
-	}
-	if (nukeicon && fw->icon_name.name)
-	{
-#ifdef CODE_WITH_LEAK_I_THINK
-		if ((fw->name.name != fw->icon_name.name || nukename) &&
-		    fw->icon_name.name != NoName)
-		{
-			FlocaleFreeNameProperty(&(fw->icon_name));
-			fw->visible_icon_name = NULL;
-		}
-#else
-		if ((fw->name.name != fw->icon_name.name) &&
-		    fw->icon_name.name != NoName)
-		{
-			FlocaleFreeNameProperty(&(fw->icon_name));
-			fw->visible_icon_name = NULL;
-		}
-#endif
 	}
 
 	return;
@@ -2973,11 +3046,11 @@ void free_window_names(FvwmWindow *fw, Bool nukename, Bool nukeicon)
 
 
 
-/***************************************************************************
+/*
  *
  * Handles destruction of a window
  *
- ****************************************************************************/
+ */
 void destroy_window(FvwmWindow *fw)
 {
 	/* Warning, this is also called by HandleUnmapNotify; if it ever needs
@@ -2988,26 +3061,44 @@ void destroy_window(FvwmWindow *fw)
 		return;
 	}
 
+	/* remove window style */
+	if (!IS_SCHEDULED_FOR_DESTROY(fw) && !DO_REUSE_DESTROYED(fw))
+	{
+		style_id_t s_id;
+
+		memset(&s_id, 0, sizeof(style_id_t));
+		SID_SET_WINDOW_ID(s_id, (XID)FW_W(fw));
+		SID_SET_HAS_WINDOW_ID(s_id, True);
+
+		style_destroy_style(s_id);
+	}
+
 	/****** remove from window list ******/
-
-	/* first of all, remove the window from the list of all windows! */
-	if (fw->prev != NULL)
+	/* if the window is sheduled fro destroy the window has been already
+	 * removed from list */
+	if (!IS_SCHEDULED_FOR_DESTROY(fw))
 	{
-		fw->prev->next = fw->next;
-	}
-	if (fw->next != NULL)
-	{
-		fw->next->prev = fw->prev;
-	}
+		/* first, remove the window from the list of all windows! */
+		if (fw->prev != NULL)
+		{
+			fw->prev->next = fw->next;
+		}
+		if (fw->next != NULL)
+		{
+			fw->next->prev = fw->prev;
+		}
+		fw->next = NULL;
+		fw->prev = NULL;
 
-	/****** also remove it from the stack ring ******/
+		/****** also remove it from the stack ring ******/
 
-	/*
-	 * RBW - 11/13/1998 - new: have to unhook the stacking order chain also.
-	 * There's always a prev and next, since this is a ring anchored on
-	 * Scr.FvwmRoot
-	 */
-	remove_window_from_stack_ring(fw);
+		/*
+		 * RBW - 11/13/1998 - new: have to unhook the stacking order
+		 chain also. There's always a prev and next, since this is a
+		 ring anchored on Scr.FvwmRoot
+		*/
+		remove_window_from_stack_ring(fw);
+	}
 
 	/****** check if we have to delay window destruction ******/
 
@@ -3015,13 +3106,17 @@ void destroy_window(FvwmWindow *fw)
 	     Scr.flags.is_executing_menu_function) &&
 	    !DO_REUSE_DESTROYED(fw))
 	{
+		if (IS_SCHEDULED_FOR_DESTROY(fw))
+		{
+			return;
+		}
 		/* mark window for destruction */
 		SET_SCHEDULED_FOR_DESTROY(fw, 1);
 		Scr.flags.is_window_scheduled_for_destroy = 1;
 		/* this is necessary in case the application destroys the
 		 * client window and a new window is created with the same
 		 * window id */
-		XDeleteContext(dpy, FW_W(fw), FvwmContext);
+		delete_client_context(fw);
 		XFlush(dpy);
 		/* unmap the the window to fake that it was already removed */
 		if (IS_ICONIFIED(fw))
@@ -3045,6 +3140,8 @@ void destroy_window(FvwmWindow *fw)
 			(unsigned long)fw);
 		EWMH_DestroyWindow(fw);
 		focus_grab_buttons_on_layer(fw->layer);
+		Scr.FWScheduledForDestroy = flist_append_obj(
+			Scr.FWScheduledForDestroy, fw);
 		return;
 	}
 
@@ -3135,14 +3232,14 @@ void destroy_window(FvwmWindow *fw)
 }
 
 
-/***********************************************************************
+/*
  *
  *  Procedure:
  *      RestoreWithdrawnLocation
  *
  *  Puts windows back where they were before fvwm took over
  *
- ************************************************************************/
+ */
 void RestoreWithdrawnLocation(
 	FvwmWindow *fw, Bool is_restart_or_recapture, Window parent)
 {
@@ -3185,8 +3282,8 @@ void RestoreWithdrawnLocation(
 	 * this happens.  The XTranslateCoordinates call above will set the
 	 * window coordinates to either be larger than the screen, or negative.
 	 * This will result in the window being placed in odd, or even
-	 * unviewable locations when the window is remapped.  The following code
-	 * forces the "relative" location to be within the bounds of the
+	 * unviewable locations when the window is remapped.  The following
+	 * code forces the "relative" location to be within the bounds of the
 	 * display.
 	 *
 	 * gpw -- 11/11/93
@@ -3257,12 +3354,12 @@ void RestoreWithdrawnLocation(
 }
 
 
-/***********************************************************************
+/*
  *
  *  Procedure:
  *      Reborder - Removes fvwm border windows
  *
- ************************************************************************/
+ */
 void Reborder(void)
 {
 	FvwmWindow *fw;
@@ -3301,7 +3398,7 @@ void CaptureAllWindows(const exec_context_t *exc, Bool is_recapture)
 	int i,j;
 	unsigned int nchildren;
 	Window root, parent, *children;
-	initial_window_options_type win_opts;
+	initial_window_options_t win_opts;
 	FvwmWindow *fw;
 
 	MyXGrabServer(dpy);
@@ -3417,7 +3514,7 @@ void CaptureAllWindows(const exec_context_t *exc, Bool is_recapture)
 	return;
 }
 
-/* ---------------------------- builtin commands ---------------------------- */
+/* ---------------------------- builtin commands --------------------------- */
 
 void CMD_Recapture(F_CMD_ARGS)
 {
