@@ -1,3 +1,4 @@
+/* -*-c-*- */
 /* This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -13,19 +14,19 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-/****************************************************************************
+/*
  * This module is all original code
  * by Rob Nation
  * Copyright 1993, Robert Nation
  *     You may use this code for any purpose, as long as the original
  *     copyright remains in the source code and all documentation
- ****************************************************************************/
+ */
 
-/***********************************************************************
+/*
  *
  * code for launching fvwm modules.
  *
- ***********************************************************************/
+ */
 #include "config.h"
 
 #include <stdio.h>
@@ -76,9 +77,9 @@ typedef struct
 	int done;
 } mqueue_object_type;
 
-msg_masks_type *PipeMask;
-static msg_masks_type *SyncMask;
-static msg_masks_type *NoGrabMask;
+msg_masks_t *PipeMask;
+static msg_masks_t *SyncMask;
+static msg_masks_t *NoGrabMask;
 fqueue *pipeQueue;
 
 extern fd_set init_fdset;
@@ -91,7 +92,7 @@ static void AddToCommandQueue(Window w, int module, char * command);
  * function operates only on the extended messages, otherwise it operates only
  * on normal messages.
  */
-static void set_message_mask(msg_masks_type *mask, unsigned long msg)
+static void set_message_mask(msg_masks_t *mask, unsigned long msg)
 {
 	if (msg & M_EXTENDED_MSG)
 	{
@@ -138,10 +139,10 @@ void initModules(void)
 	writePipes = (int *)safemalloc(sizeof(int)*npipes);
 	readPipes = (int *)safemalloc(sizeof(int)*npipes);
 	pipeOn = (int *)safemalloc(sizeof(int)*npipes);
-	PipeMask = (msg_masks_type *)safemalloc(sizeof(msg_masks_type)*npipes);
-	SyncMask = (msg_masks_type *)safemalloc(sizeof(msg_masks_type)*npipes);
-	NoGrabMask = (msg_masks_type *)safemalloc(
-		sizeof(msg_masks_type)*npipes);
+	PipeMask = (msg_masks_t *)safemalloc(sizeof(msg_masks_t)*npipes);
+	SyncMask = (msg_masks_t *)safemalloc(sizeof(msg_masks_t)*npipes);
+	NoGrabMask = (msg_masks_t *)safemalloc(
+		sizeof(msg_masks_t)*npipes);
 	pipeName = (char **)safemalloc(sizeof(char *)*npipes);
 	pipeAlias = (char **)safemalloc(sizeof(char *)*npipes);
 	pipeQueue = (fqueue *)safemalloc(sizeof(fqueue)*npipes);
@@ -409,15 +410,15 @@ static int do_execute_module(F_CMD_ARGS, Bool desperate)
 #endif
 		if (!Pdefault)
 		{
-			char *visualid, *colormap;
+			char visualid[32];
+			char colormap[32];
 
-			visualid = safemalloc(32);
-			sprintf(visualid, "FVWM_VISUALID=%lx",
+			sprintf(
+				visualid, "FVWM_VISUALID=%lx",
 				XVisualIDFromVisual(Pvisual));
-			putenv(visualid);
-			colormap = safemalloc(32);
+			flib_putenv("FVWM_VISUALID", visualid);
 			sprintf(colormap, "FVWM_COLORMAP=%lx", Pcmap);
-			putenv(colormap);
+			flib_putenv("FVWM_COLORMAP", colormap);
 		}
 
 		/* Why is this execvp??  We've already searched the module
@@ -643,6 +644,7 @@ static void ExecuteModuleCommand(Window w, int module, char *text)
 	XEvent e;
 	const exec_context_t *exc;
 	exec_context_changes_t ecc;
+	int flags;
 
 	if (XFindContext(dpy, w, FvwmContext, (caddr_t *)&ecc.w.fw) == XCNOENT)
 	{
@@ -684,6 +686,7 @@ static void ExecuteModuleCommand(Window w, int module, char *text)
 	fev_fake_event(&e);
 	ecc.type = EXCT_MODULE;
 	ecc.w.w = w;
+	flags = (w == None) ? 0 : FUNC_DONT_DEFER;
 	ecc.w.wcontext = GetContext(NULL, ecc.w.fw, &e, &w);
 	ecc.x.etrigger = &e;
 	ecc.m.modnum = module;
@@ -782,6 +785,7 @@ Bool HandleModuleInput(Window w, int module, char *expect, Bool queue)
 
 RETSIGTYPE DeadPipe(int sig)
 {
+	SIGNAL_RETURN;
 }
 
 void KillModule(int channel)
@@ -852,7 +856,10 @@ void CMD_KillModule(F_CMD_ARGS)
 	GetNextToken(action, &alias);
 	KillModuleByName(module, alias);
 	free(module);
-
+	if (alias)
+	{
+		free(alias);
+	}
 	return;
 }
 
@@ -882,7 +889,7 @@ make_vpacket(unsigned long *body, unsigned long event_type,
 
 
 
-/*************************************************************
+/*
     RBW - 04/16/1999 - new packet builder for GSFR --
     Arguments are pairs of lengths and argument data pointers.
     RBW - 05/01/2000 -
@@ -891,7 +898,7 @@ make_vpacket(unsigned long *body, unsigned long event_type,
     a special hack to accommodate the old CONFIGARGSNEW
     technique of sending the args for the M_CONFIGURE_WINDOW
     packet.
-**************************************************************/
+*/
 static unsigned long
 make_new_vpacket(unsigned char *body, unsigned long event_type,
 		 unsigned long num, va_list ap)
@@ -992,9 +999,9 @@ BroadcastPacket(unsigned long event_type, unsigned long num_datum, ...)
 }
 
 
-/* ************************************************************
+/*
    RBW - 04/16/1999 - new style packet senders for GSFR --
-   ************************************************************ */
+*/
 static void SendNewPacket(int module, unsigned long event_type,
 			  unsigned long num_datum, ...)
 {
@@ -1132,7 +1139,7 @@ static void BroadcastNewPacket(unsigned long event_type,
 
 
 
-/****************************************************************
+/*
     RBW - 04/16/1999 - new version for GSFR --
 	- args are now pairs:
 	  - length of arg data
@@ -1141,7 +1148,7 @@ static void BroadcastNewPacket(unsigned long event_type,
 	- the 9th field, where flags used to be, is temporarily left
 	as a dummy to preserve alignment of the other fields in the
 	old packet: we should drop this before the next release.
-*****************************************************************/
+*/
 #define CONFIGARGSNEW(_t) 28,\
 	    (unsigned long)(sizeof(unsigned long)),\
 	    &FW_W(*(_t)),\
@@ -1437,9 +1444,9 @@ BroadcastFvwmPicture(
 	return;
 }
 
-/**********************************************************************
+/*
  * Reads a colorset command from a module and broadcasts it back out
- **********************************************************************/
+ */
 void BroadcastColorset(int n)
 {
 	int i;
@@ -1458,9 +1465,9 @@ void BroadcastColorset(int n)
 	return;
 }
 
-/**********************************************************************
+/*
  * Broadcasts a string to all modules as M_CONFIG_INFO.
- **********************************************************************/
+ */
 void BroadcastPropertyChange(unsigned long argument, unsigned long data1,
 			     unsigned long data2, char *string)
 {
@@ -1480,9 +1487,9 @@ void BroadcastPropertyChange(unsigned long argument, unsigned long data1,
 	return;
 }
 
-/**********************************************************************
+/*
  * Broadcasts a string to all modules as M_CONFIG_INFO.
- **********************************************************************/
+ */
 void BroadcastConfigInfoString(char *string)
 {
 	int i;
@@ -1500,9 +1507,9 @@ void BroadcastConfigInfoString(char *string)
 }
 
 
-/**********************************************************************
+/*
  * Broadcasts the state of Xinerama support to all modules as M_CONFIG_INFO.
- **********************************************************************/
+ */
 void broadcast_xinerama_state(void)
 {
 	BroadcastConfigInfoString((char *)FScreenGetConfiguration());
@@ -1511,9 +1518,9 @@ void broadcast_xinerama_state(void)
 }
 
 
-/**********************************************************************
+/*
  * Broadcasts the ignored modifiers to all modules as M_CONFIG_INFO.
- **********************************************************************/
+ */
 void broadcast_ignore_modifiers(void)
 {
 	char msg[32];
@@ -1584,7 +1591,7 @@ extern char *ModuleUnlock;              /* defined in libs/Module.c */
 void PositiveWrite(int module, unsigned long *ptr, int size)
 {
 	extern int moduleTimeout;
-	msg_masks_type mask;
+	msg_masks_t mask;
 
 	if (ptr == NULL)
 	{
@@ -1741,8 +1748,8 @@ static void DeleteMessageQueueBuff(int module)
 	{
 		/* remove from queue */
 		fqueue_remove_or_operate_from_front(
-			&pipeQueue[module], NULL, NULL);
-		/* we don't need to free the ocj->data here because it's in the
+			&pipeQueue[module], NULL, NULL, NULL, NULL);
+		/* we don't need to free the obj->data here because it's in the
 		 * same malloced block as the obj itself. */
 		free(obj);
 	}
@@ -1774,18 +1781,13 @@ void FlushMessageQueue(int module)
 			{
 				obj->done += a;
 			}
-			else if (errno == EINTR)
-			{
-				return;
-			}
 			/* the write returns EWOULDBLOCK or EAGAIN if the pipe
 			 * is full. (This is non-blocking I/O). SunOS returns
 			 * EWOULDBLOCK, OSF/1 returns EAGAIN under these
 			 * conditions. Hopefully other OSes return one of these
 			 * values too. Solaris 2 doesn't seem to have a man
 			 * page for write(2) (!) */
-			else if ((errno == EWOULDBLOCK) || (errno == EAGAIN) ||
-				 (errno==EINTR))
+			else if (errno == EWOULDBLOCK || errno == EAGAIN)
 			{
 				fd_set writeSet;
 				struct timeval timeout;
@@ -1834,7 +1836,7 @@ void FlushMessageQueue(int module)
 				/* pipe accepts further input; continue */
 				continue;
 			}
-			else
+			else if (errno != EINTR)
 			{
 				KillModule(module);
 				return;
@@ -1871,12 +1873,12 @@ typedef struct
 
 static fqueue cqueue = FQUEUE_INIT;
 
-/***********************************************************************
+/*
  *
  *  Procedure:
  *      AddToCommandQueue - add a module command to the command queue
  *
- ************************************************************************/
+ */
 
 static void AddToCommandQueue(Window window, int module, char *command)
 {
@@ -1897,14 +1899,14 @@ static void AddToCommandQueue(Window window, int module, char *command)
 	return;
 }
 
-/***********************************************************************
+/*
  *
  *  Procedure:
  *      ExecuteCommandQueue - runs command from the module command queue
  *      This may be called recursively if a module command runs a function
  *      that does a Wait, so it must be re-entrant
  *
- ************************************************************************/
+ */
 void ExecuteCommandQueue(void)
 {
 	cqueue_object_type *obj;
@@ -1912,7 +1914,8 @@ void ExecuteCommandQueue(void)
 	while (fqueue_get_first(&cqueue, (void **)&obj) == 1)
 	{
 		/* remove from queue */
-		fqueue_remove_or_operate_from_front(&cqueue, NULL, NULL);
+		fqueue_remove_or_operate_from_front(
+			&cqueue, NULL, NULL, NULL, NULL);
 		/* execute and destroy */
 		if (obj->command)
 		{
