@@ -1,3 +1,4 @@
+/* -*-c-*- */
 /* This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -13,19 +14,19 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-/*****************************************************************************
+/*
  * This module is all original code
  * by Rob Nation
  * Copyright 1993, Robert Nation
  *     You may use this code for any purpose, as long as the original
  *     copyright remains in the source code and all documentation
- ****************************************************************************/
+ */
 
-/************************************************************************
+/*
  *
  * code for moving and resizing windows
  *
- ***********************************************************************/
+ */
 
 #include "config.h"
 
@@ -57,6 +58,8 @@
 #include "eventhandler.h"
 #include "eventmask.h"
 #include "colormaps.h"
+#include "update.h"
+#include "stack.h"
 
 /* ----- move globals ----- */
 
@@ -102,7 +105,7 @@ static void draw_move_resize_grid(int x, int  y, int  width, int height);
 
 /* ----- end of resize globals ----- */
 
-/***********************************************************************
+/*
  *
  *  Procedure:
  *      draw_move_resize_grid - move a window outline
@@ -114,7 +117,7 @@ static void draw_move_resize_grid(int x, int  y, int  width, int height);
  *      width       - the width of the rectangle
  *      height      - the height of the rectangle
  *
- ***********************************************************************/
+ */
 static int get_outline_rects(
 	XRectangle *rects, int x, int y, int width, int height)
 {
@@ -140,29 +143,29 @@ static int get_outline_rects(
 
 	for (i = 0; i < n; i++)
 	{
-		rects[i+i].x = x + i;
-		rects[i+i].y = y + i;
-		rects[i+i].width = width - (i << 1);
-		rects[i+i].height = height - (i << 1);
+		rects[i].x = x + i;
+		rects[i].y = y + i;
+		rects[i].width = width - (i << 1);
+		rects[i].height = height - (i << 1);
 	}
 	if (width - (n << 1) >= 5 && height - (n << 1) >= 5)
 	{
 		if (width - (n << 1) >= 10)
 		{
 			int off = (width - (n << 1)) / 3 + n;
-			rects[i+i].x = x + off;
-			rects[i+i].y = y + n;
-			rects[i+i].width = width - (off << 1);
-			rects[i+i].height = height - (n << 1);
+			rects[i].x = x + off;
+			rects[i].y = y + n;
+			rects[i].width = width - (off << 1);
+			rects[i].height = height - (n << 1);
 			i++;
 		}
 		if (height - (n << 1) >= 10)
 		{
 			int off = (height - (n << 1)) / 3 + n;
-			rects[i+i].x = x + n;
-			rects[i+i].y = y + off;
-			rects[i+i].width = width - (n << 1);
-			rects[i+i].height = height - (off << 1);
+			rects[i].x = x + n;
+			rects[i].y = y + off;
+			rects[i].width = width - (n << 1);
+			rects[i].height = height - (off << 1);
 			i++;
 		}
 	}
@@ -207,7 +210,7 @@ static void draw_move_resize_grid(int x, int  y, int  width, int height)
 		move_resize_grid.flags.is_enabled = 0;
 		nrects +=
 			get_outline_rects(
-				&(rects[1]), move_resize_grid.geom.x,
+				&(rects[0]), move_resize_grid.geom.x,
 				move_resize_grid.geom.y,
 				move_resize_grid.geom.width,
 				move_resize_grid.geom.height);
@@ -219,11 +222,12 @@ static void draw_move_resize_grid(int x, int  y, int  width, int height)
 		move_resize_grid.geom.y = y;
 		move_resize_grid.geom.width = width;
 		move_resize_grid.geom.height = height;
-		nrects += get_outline_rects(&(rects[0]), x, y, width, height);
+		nrects += get_outline_rects(
+			&(rects[nrects]), x, y, width, height);
 	}
 	if (nrects > 0)
 	{
-		XDrawRectangles(dpy, Scr.Root, Scr.XorGC, rects, 10);
+		XDrawRectangles(dpy, Scr.Root, Scr.XorGC, rects, nrects);
 		XFlush(dpy);
 	}
 
@@ -248,7 +252,8 @@ void switch_move_resize_grid(Bool state)
 	}
 	else if (!move_resize_grid.flags.is_enabled)
 	{
-		if (move_resize_grid.geom.width && move_resize_grid.geom.height)
+		if (move_resize_grid.geom.width &&
+		    move_resize_grid.geom.height)
 		{
 			draw_move_resize_grid(
 				move_resize_grid.geom.x,
@@ -304,7 +309,7 @@ static int GetOnePositionArgument(
 	{
 		*pFinalX = max-w - (int)(val*factor);
 	}
-	else if (sscanf(s1,"%d",&val) == 1)
+	else if (sscanf(s1,"+%d",&val) == 1 || sscanf(s1,"%d",&val) == 1)
 	{
 		*pFinalX = (int)(val*factor);
 	}
@@ -365,17 +370,14 @@ static int GetMoveArguments(
 		free(s1);
 		return 0;
 	}
-	else
+	action = GetNextToken(action, &s2);
+	if (fWarp)
 	{
-		action = GetNextToken(action, &s2);
-		if (fWarp)
+		warp = PeekToken(action, &naction);
+		if (StrEquals(warp, "Warp"))
 		{
-			warp = PeekToken(action, &naction);
-			if (StrEquals(warp, "Warp"))
-			{
-				*fWarp = True;
-				action = naction;
-			}
+			*fWarp = True;
+			action = naction;
 		}
 	}
 
@@ -642,7 +644,7 @@ void resize_geometry_window(void)
 	return;
 }
 
-/***********************************************************************
+/*
  *
  *  Procedure:
  *      DisplayPosition - display the position in the dimensions window
@@ -651,7 +653,7 @@ void resize_geometry_window(void)
  *      tmp_win - the current fvwm window
  *      x, y    - position of the window
  *
- ************************************************************************/
+ */
 
 static void DisplayPosition(
 	const FvwmWindow *tmp_win, const XEvent *eventp, int x, int y,int Init)
@@ -714,7 +716,7 @@ static void DisplayPosition(
 }
 
 
-/***********************************************************************
+/*
  *
  *  Procedure:
  *      DisplaySize - display the size in the dimensions window
@@ -724,7 +726,7 @@ static void DisplayPosition(
  *      width   - the width of the rubber band
  *      height  - the height of the rubber band
  *
- ***********************************************************************/
+ */
 static void DisplaySize(
 	const FvwmWindow *tmp_win, const XEvent *eventp, int width,
 	int height, Bool Init, Bool resetLast)
@@ -901,6 +903,14 @@ static Bool resize_move_window(F_CMD_ARGS)
 
 void CMD_ResizeMove(F_CMD_ARGS)
 {
+	FvwmWindow *fw = exc->w.fw;
+
+	if (IS_EWMH_FULLSCREEN(fw))
+	{
+		/* do not unmaximize ! */
+		CMD_ResizeMoveMaximize(F_PASS_ARGS);
+		return;
+	}
 	resize_move_window(F_PASS_ARGS);
 
 	return;
@@ -1048,6 +1058,7 @@ static void AnimatedMoveAnyWindow(
 	int deltaX, deltaY;
 	Bool first = True;
 	XEvent evdummy;
+	unsigned int draw_parts = PART_NONE;
 
 	if (!is_function_allowed(F_MOVE, NULL, fw, True, False))
 	{
@@ -1093,6 +1104,11 @@ static void AnimatedMoveAnyWindow(
 		return;
 	}
 
+	if (fw && w == FW_W_FRAME(fw))
+	{
+		draw_parts = border_get_transparent_decorations_part(fw);
+	}
+
 	/* Needed for aborting */
 	MyXGrabKeyboard(dpy);
 	do
@@ -1111,6 +1127,18 @@ static void AnimatedMoveAnyWindow(
 				pmrtp, first,
 				currentX, currentY, endX, endY);
 		}
+		else if (draw_parts != PART_NONE)
+		{
+			border_draw_decorations(
+				fw, draw_parts,
+				((fw == get_focus_window())) ?
+				True : False,
+				True, CLEAR_ALL, NULL, NULL);
+		}
+		if (fw && pmrtp == NULL && IS_TEAR_OFF_MENU(fw))
+		{
+			menu_redraw_transparent_tear_off_menu(fw, False);
+		}
 		if (fWarpPointerToo == True)
 		{
 			if (FQueryPointer(
@@ -1131,7 +1159,7 @@ static void AnimatedMoveAnyWindow(
 				dpy, None, Scr.Root, 0, 0, 0, 0, pointerX,
 				pointerY);
 		}
-		if (fw && !IS_SHADED(fw))
+		if (fw && !IS_SHADED(fw) && !Scr.bo.DisableConfigureNotify)
 		{
 			/* send configure notify event for windows that care
 			 * about their location */
@@ -1218,11 +1246,11 @@ void AnimatedMoveFvwmWindow(
 	return;
 }
 
-/****************************************************************************
+/*
  *
  * Start a window move operation
  *
- ****************************************************************************/
+ */
 void __move_icon(
 	FvwmWindow *fw, int x, int y, int old_x, int old_y,
 	Bool do_move_animated, Bool do_warp_pointer)
@@ -1259,25 +1287,24 @@ void __move_icon(
 	{
 		AnimatedMoveOfWindow(
 			tw, -1, -1, tx, ty, do_warp_pointer, -1, NULL, NULL);
+		do_warp_pointer = 0;
 	}
-	else
+	if (has_icon_title)
 	{
-		XMoveWindow(dpy, FW_W_ICON_TITLE(fw), tx, ty);
-		if (do_warp_pointer)
-		{
-			FWarpPointer(
-				dpy, None, None, 0, 0, 0, 0, x - old_x,
-				y - old_y);
-		}
+		XMoveWindow(dpy, FW_W_ICON_TITLE(fw), gt.x, gt.y);
 	}
 	if (has_icon_picture)
 	{
+		XMoveWindow(dpy, FW_W_ICON_PIXMAP(fw), gp.x, gp.y);
+		XMapWindow(dpy, FW_W_ICON_PIXMAP(fw));
 		if (has_icon_title)
 		{
-			XMoveWindow(dpy, FW_W_ICON_TITLE(fw), gt.x, gt.y);
 			XMapWindow(dpy, FW_W_ICON_TITLE(fw));
 		}
-		XMapWindow(dpy, tw);
+	}
+	if (do_warp_pointer)
+	{
+		FWarpPointer(dpy, None, None, 0, 0, 0, 0, x - old_x, y - old_y);
 	}
 
 	return;
@@ -1322,14 +1349,16 @@ static void __move_window(F_CMD_ARGS, Bool do_animate, int mode)
 	{
 		return;
 	}
-
+	if (mode == MOVE_PAGE && IS_STICKY_ACROSS_PAGES(fw))
+	{
+		return;
+	}
 	if (mode == MOVE_PAGE)
 	{
 		rectangle r;
 		rectangle s;
 
 		do_animate = False;
-
 		if (!get_page_arguments(action, &page_x, &page_y))
 		{
 			page_x = Scr.Vx;
@@ -1355,8 +1384,6 @@ static void __move_window(F_CMD_ARGS, Bool do_animate, int mode)
 		int fscreen;
 
 		do_animate = False;
-		SET_STICKY(fw, 0);
-
 		fscreen = FScreenGetScreenArgument(
 			action, FSCREEN_SPEC_CURRENT);
 		FScreenGetScrRect(
@@ -1804,7 +1831,7 @@ static void DoSnapAttract(
 			&scr_y1);
 
 		/* snap to right edge */
-		if (scr_x1 != Scr.MyDisplayWidth  &&
+		if (scr_x1 < Scr.MyDisplayWidth &&
 		    *px + Width >= scr_x1 && *px + Width <
 		    scr_x1 + Scr.XiMoveResistance)
 		{
@@ -1812,7 +1839,7 @@ static void DoSnapAttract(
 			do_recalc_rectangle = True;
 		}
 		/* snap to left edge */
-		else if (scr_x0 != 0 &&
+		else if (scr_x0 > 0 &&
 			 *px <= scr_x0 && scr_x0 - *px < Scr.XiMoveResistance)
 		{
 			*px = scr_x0;
@@ -1828,14 +1855,14 @@ static void DoSnapAttract(
 				&scr_x1, &scr_y1);
 		}
 		/* snap to bottom edge */
-		if (scr_y1 != Scr.MyDisplayHeight &&
+		if (scr_y1 < Scr.MyDisplayHeight &&
 		    *py + Height >= scr_y1 && *py + Height <
 		    scr_y1 + Scr.XiMoveResistance)
 		{
 			*py = scr_y1 - Height;
 		}
 		/* snap to top edge */
-		else if (scr_y0 != 0 &&
+		else if (scr_y0 > 0 &&
 			 *py <= scr_y0 && scr_y0 - *py < Scr.XiMoveResistance)
 		{
 			*py = scr_y0;
@@ -1845,13 +1872,13 @@ static void DoSnapAttract(
 	return;
 }
 
-/****************************************************************************
+/*
  *
  * Move the rubberband around, return with the new window location
  *
  * Returns True if the window has to be resized after the move.
  *
- ****************************************************************************/
+ */
 Bool __move_loop(
 	const exec_context_t *exc, int XOffset, int YOffset, int Width,
 	int Height, int *FinalX, int *FinalY,Bool do_move_opaque)
@@ -1885,6 +1912,8 @@ Bool __move_loop(
 	 * released */
 	Bool nosnap_enabled = False;
 	FvwmWindow *fw = exc->w.fw;
+	unsigned int draw_parts = PART_NONE;
+	XEvent e;
 
 	if (!GrabEm(CRS_MOVE, GRAB_NORMAL))
 	{
@@ -1956,13 +1985,21 @@ Bool __move_loop(
 		draw_move_resize_grid(xl, yt, Width - 1, Height - 1);
 	}
 
+	if (move_w == FW_W_FRAME(fw) && do_move_opaque)
+	{
+		draw_parts = border_get_transparent_decorations_part(fw);
+	}
 	DisplayPosition(fw, exc->x.elast, xl, yt, True);
 
+	memset(&e, 0, sizeof(e));
 	while (!finished && bad_window != FW_W(fw))
 	{
 		int rc = 0;
-		XEvent e;
+		int old_xl;
+		int old_yt;
 
+		old_xl = xl;
+		old_yt = yt;
 		/* wait until there is an interesting event */
 		while (rc != -1 &&
 		       (!FPending(dpy) ||
@@ -2181,7 +2218,9 @@ Bool __move_loop(
 				continue;
 			}
 			if (!(e.xkey.state & Mod1Mask))
+			{
 				nosnap_enabled = True;
+			}
 			do_snap = nosnap_enabled &&
 				(e.xkey.state & Mod1Mask) ? False : True;
 
@@ -2277,7 +2316,6 @@ Bool __move_loop(
 				}
 				paged++;
 			}  /* end while (paged) */
-
 			break;
 
 		case Expose:
@@ -2302,7 +2340,7 @@ Bool __move_loop(
 		xl += x_virtual_offset;
 		yt += y_virtual_offset;
 		if (do_move_opaque && !IS_ICONIFIED(fw) &&
-		    !IS_SHADED(fw))
+		    !IS_SHADED(fw) && !Scr.bo.DisableConfigureNotify)
 		{
 			/* send configure notify event for windows that care
 			 * about their location; don't send anything if
@@ -2330,10 +2368,26 @@ Bool __move_loop(
 				fw_copy.frame_g.x = xl;
 				fw_copy.frame_g.y = yt;
 			}
-			/* only do this with opaque moves, (i.e. the server is
-			 * not grabbed) */
-			BroadcastConfig(M_CONFIGURE_WINDOW, &fw_copy);
-			FlushAllMessageQueues();
+			if (xl != old_xl || yt != old_yt)
+			{
+				/* only do this with opaque moves, (i.e. the
+				 * server is not grabbed) */
+				if (draw_parts != PART_NONE)
+				{
+					border_draw_decorations(
+						fw, draw_parts,
+						((fw == get_focus_window())) ?
+						True : False,
+						True, CLEAR_ALL, NULL, NULL);
+				}
+				if (IS_TEAR_OFF_MENU(fw))
+				{
+					menu_redraw_transparent_tear_off_menu(
+						fw, False);
+				}
+				BroadcastConfig(M_CONFIGURE_WINDOW, &fw_copy);
+				FlushAllMessageQueues();
+			}
 		}
 	} /* while (!finished) */
 
@@ -2350,6 +2404,15 @@ Bool __move_loop(
 		if (aborted && do_move_opaque)
 		{
 			XMoveWindow(dpy, move_w, x_bak, y_bak);
+			if (draw_parts != PART_NONE)
+			{
+				border_draw_decorations(
+					fw, draw_parts,
+					((fw == get_focus_window())) ?
+					True : False,
+					True, CLEAR_ALL, NULL, NULL);
+			}
+			menu_redraw_transparent_tear_off_menu(fw, False);
 		}
 		if (bad_window == FW_W(fw))
 		{
@@ -2660,7 +2723,7 @@ void CMD_XorPixmap(F_CMD_ARGS)
 
 /* ----------------------------- resizing code ----------------------------- */
 
-/***********************************************************************
+/*
  *
  *  Procedure:
  *      __resize_step - move the rubberband around.  This is called for
@@ -2675,7 +2738,7 @@ void CMD_XorPixmap(F_CMD_ARGS)
  *      xmotionp - pointer to xmotion in resize_window
  *      ymotionp - pointer to ymotion in resize_window
  *
- ************************************************************************/
+ */
 static void __resize_step(
 	const exec_context_t *exc, int x_root, int y_root, rectangle *drag,
 	rectangle *orig, int *xmotionp, int *ymotionp, Bool do_resize_opaque)
@@ -2784,6 +2847,7 @@ static Bool __resize_window(F_CMD_ARGS)
 	frame_move_resize_args mr_args = NULL;
 	long evmask;
 	FvwmWindow *fw = exc->w.fw;
+	XEvent ev;
 
 	bad_window = False;
 	ResizeWindow = FW_W_FRAME(fw);
@@ -3125,17 +3189,18 @@ static Bool __resize_window(F_CMD_ARGS)
 	}
 
 	/* loop to resize */
+	memset(&ev, 0, sizeof(ev));
 	while (!finished && bad_window != FW_W(fw))
 	{
 		int rc = 0;
-		XEvent ev;
 
 		/* block until there is an interesting event */
-		while (rc != -1 && !FCheckMaskEvent(dpy, evmask, &ev))
+		while (rc != -1 &&
+		       (!FPending(dpy) || !FCheckMaskEvent(dpy, evmask, &ev)))
 		{
 			rc = HandlePaging(
-				&ev, Scr.EdgeScrollX, Scr.EdgeScrollY, &x,
-				&y, &delta_x, &delta_y, False, False, True);
+				&ev, Scr.EdgeScrollX, Scr.EdgeScrollY, &x, &y,
+				&delta_x, &delta_y, False, False, True);
 			if (rc == 1)
 			{
 				/* Fake an event to force window reposition */
@@ -3264,6 +3329,7 @@ static Bool __resize_window(F_CMD_ARGS)
 			}
 			fForceRedraw = False;
 			is_done = True;
+			break;
 
 		case PropertyNotify:
 		{
@@ -3274,6 +3340,7 @@ static Bool __resize_window(F_CMD_ARGS)
 			ea.exc = exc_clone_context(exc, &ecc, ECC_ETRIGGER);
 			HandlePropertyNotify(&ea);
 			exc_destroy_context(ea.exc);
+			is_done = True;
 			break;
 		}
 
@@ -3283,9 +3350,11 @@ static Bool __resize_window(F_CMD_ARGS)
 		if (!is_done)
 		{
 			if (!do_resize_opaque)
+			{
 				/* must undraw the rubber band in case the
 				 * event causes some drawing */
 				switch_move_resize_grid(False);
+			}
 			dispatch_event(&ev);
 			if (!do_resize_opaque)
 			{
@@ -3403,6 +3472,15 @@ static Bool __resize_window(F_CMD_ARGS)
 
 void CMD_Resize(F_CMD_ARGS)
 {
+	FvwmWindow *fw = exc->w.fw;
+
+	if (IS_EWMH_FULLSCREEN(fw))
+	{
+		/* do not unmaximize ! */
+		CMD_ResizeMaximize(F_PASS_ARGS);
+		return;
+	}
+
 	__resize_window(F_PASS_ARGS);
 
 	return;
@@ -3410,6 +3488,32 @@ void CMD_Resize(F_CMD_ARGS)
 
 /* ----------------------------- maximizing code --------------------------- */
 
+Bool is_window_sticky_across_pages(FvwmWindow *fw)
+{
+	if (IS_STICKY_ACROSS_PAGES(fw) ||
+	    (IS_ICONIFIED(fw) && IS_ICON_STICKY_ACROSS_PAGES(fw)))
+	{
+		return True;
+	}
+	else
+	{
+		return False;
+	}
+}
+
+Bool is_window_sticky_across_desks(FvwmWindow *fw)
+{
+	if (IS_STICKY_ACROSS_DESKS(fw) ||
+	    (IS_ICONIFIED(fw) && IS_ICON_STICKY_ACROSS_DESKS(fw)))
+	{
+		return True;
+	}
+	else
+	{
+		return False;
+	}
+}
+
 static void move_sticky_window_to_same_page(
 	int *x11, int *x12, int *y11, int *y12,
 	int x21, int x22, int y21, int y22)
@@ -3463,7 +3567,6 @@ static void MaximizeHeight(
 	int x11, x12, x21, x22;
 	int y11, y12, y21, y22;
 	int new_y1, new_y2;
-	Bool is_sticky;
 	rectangle g;
 	Bool rc;
 
@@ -3476,16 +3579,9 @@ static void MaximizeHeight(
 
 	for (cwin = Scr.FvwmRoot.next; cwin; cwin = cwin->next)
 	{
-		if (IS_STICKY(cwin) ||
-		    (IS_ICONIFIED(cwin) && IS_ICON_STICKY(cwin)))
-		{
-			is_sticky = True;
-		}
-		else
-		{
-			is_sticky = False;
-		}
-		if (cwin == win || (cwin->Desk != win->Desk && !is_sticky) ||
+		if (cwin == win ||
+		    (cwin->Desk != win->Desk &&
+		     !is_window_sticky_across_desks(cwin)) ||
 		    (layer_grow && cwin->layer <= win->layer))
 		{
 			continue;
@@ -3499,7 +3595,7 @@ static void MaximizeHeight(
 		y21 = g.y;
 		x22 = x21 + g.width;
 		y22 = y21 + g.height;
-		if (is_sticky)
+		if (is_window_sticky_across_pages(cwin))
 		{
 			move_sticky_window_to_same_page(
 				&x21, &x22, &new_y1, &new_y2, x11, x12, y11,
@@ -3542,7 +3638,6 @@ static void MaximizeWidth(
 	int x11, x12, x21, x22;
 	int y11, y12, y21, y22;
 	int new_x1, new_x2;
-	Bool is_sticky;
 	rectangle g;
 	Bool rc;
 
@@ -3555,16 +3650,9 @@ static void MaximizeWidth(
 
 	for (cwin = Scr.FvwmRoot.next; cwin; cwin = cwin->next)
 	{
-		if (IS_STICKY(cwin) ||
-		    (IS_ICONIFIED(cwin) && IS_ICON_STICKY(cwin)))
-		{
-			is_sticky = True;
-		}
-		else
-		{
-			is_sticky = False;
-		}
-		if (cwin == win || (cwin->Desk != win->Desk && !is_sticky) ||
+		if (cwin == win ||
+		    (cwin->Desk != win->Desk &&
+		     !is_window_sticky_across_desks(cwin)) ||
 		    (layer_grow && cwin->layer <= win->layer))
 		{
 			continue;
@@ -3578,7 +3666,7 @@ static void MaximizeWidth(
 		y21 = g.y;
 		x22 = x21 + g.width;
 		y22 = y21 + g.height;
-		if (is_sticky)
+		if (is_window_sticky_across_pages(cwin))
 		{
 			move_sticky_window_to_same_page(
 				&new_x1, &new_x2, &y21, &y22, x11, x12, y11,
@@ -3625,7 +3713,15 @@ static void unmaximize_fvwm_window(
 	}
 	frame_setup_window(
 		fw, new_g.x, new_g.y, new_g.width, new_g.height, True);
-
+	if (IS_EWMH_FULLSCREEN(fw))
+	{
+		SET_EWMH_FULLSCREEN(fw, False);
+		if (DO_EWMH_USE_STACKING_HINTS(fw))
+		{
+			new_layer(fw, fw->ewmh_normal_layer);
+		}
+		apply_decor_change(fw);
+	}
 	return;
 }
 
@@ -3655,18 +3751,18 @@ static void maximize_fvwm_window(
 	fw->max_offset.x = fw->normal_g.x - fw->max_g.x;
 	fw->max_offset.y = fw->normal_g.y - fw->max_g.y;
 #if 0
-fprintf(stderr,"%d %d %d %d, max_offset.x = %d, max_offset.y = %d\n", fw->max_g.x, fw->max_g.y, fw->max_g.width, fw->max_g.height, fw->max_offset.x, fw->max_offset.y);
+fprintf(stderr,"%d %d %d %d, max_offset.x = %d, max_offset.y = %d, %d %d %d %d\n", fw->max_g.x, fw->max_g.y, fw->max_g.width, fw->max_g.height, fw->max_offset.x, fw->max_offset.y, fw->normal_g.x,  fw->normal_g.y, fw->normal_g.width, fw->normal_g.height);
 #endif
 
     return;
 }
 
-/***********************************************************************
+/*
  *
  *  Procedure:
  *      (Un)Maximize a window.
  *
- ***********************************************************************/
+ */
 void CMD_Maximize(F_CMD_ARGS)
 {
 	int page_x, page_y;
@@ -3748,26 +3844,7 @@ void CMD_Maximize(F_CMD_ARGS)
 	new_g.y = fw->frame_g.y;
 	new_g.width = fw->frame_g.width;
 	new_g.height = fw->frame_g.height;
-	if (IsRectangleOnThisPage(&fw->frame_g, fw->Desk))
-	{
-		/* maximize on visible page if any part of the window is
-		 * visible */
-		page_x = 0;
-		page_y = 0;
-	}
-	else
-	{
-		int xoff = Scr.Vx % Scr.MyDisplayWidth;
-		int yoff = Scr.Vy % Scr.MyDisplayHeight;
-
-		/* maximize on the page where the center of the window is */
-		page_x = truncate_to_multiple(
-			fw->frame_g.x + fw->frame_g.width / 2 + xoff,
-			Scr.MyDisplayWidth) - xoff;
-		page_y = truncate_to_multiple(
-			fw->frame_g.y + fw->frame_g.height / 2 + yoff,
-			Scr.MyDisplayHeight) - yoff;
-	}
+	get_page_offset_check_visible(&page_x, &page_y, fw);
 
 	/* Check if we should constrain rectangle to some Xinerama screen */
 	if (!is_screen_given)
@@ -3945,12 +4022,12 @@ void CMD_Maximize(F_CMD_ARGS)
 	return;
 }
 
-/****************************************************************************
+/*
  *
  * Same as CMD_Resize and CMD_ResizeMove, but the window ends up maximized
  * without touching the normal geometry.
  *
- ****************************************************************************/
+ */
 void CMD_ResizeMaximize(F_CMD_ARGS)
 {
 	rectangle normal_g;
@@ -3973,6 +4050,7 @@ void CMD_ResizeMaximize(F_CMD_ARGS)
 		/* and mark it as maximized */
 		maximize_fvwm_window(fw, &max_g);
 	}
+	EWMH_SetWMState(fw, False);
 
 	return;
 }
@@ -3999,24 +4077,52 @@ void CMD_ResizeMoveMaximize(F_CMD_ARGS)
 		/* and mark it as maximized */
 		maximize_fvwm_window(fw, &max_g);
 	}
+	EWMH_SetWMState(fw, False);
 
 	return;
 }
 
 /* ----------------------------- stick code -------------------------------- */
 
-void handle_stick(F_CMD_ARGS, int toggle, int do_not_draw, int do_silently)
+int stick_across_pages(F_CMD_ARGS, int toggle)
 {
 	FvwmWindow *fw = exc->w.fw;
 
-	if ((toggle == 1 && IS_STICKY(fw)) || (toggle == 0 && !IS_STICKY(fw)))
+	if ((toggle == 1 && IS_STICKY_ACROSS_PAGES(fw)) ||
+	    (toggle == 0 && !IS_STICKY_ACROSS_PAGES(fw)))
 	{
-		return;
+		return 0;
+	}
+	if (IS_STICKY_ACROSS_PAGES(fw))
+	{
+		SET_STICKY_ACROSS_PAGES(fw, 0);
+	}
+	else
+	{
+		if (!IsRectangleOnThisPage(&fw->frame_g, Scr.CurrentDesk))
+		{
+			action = "";
+			__move_window(F_PASS_ARGS, FALSE, MOVE_PAGE);
+		}
+		SET_STICKY_ACROSS_PAGES(fw, 1);
 	}
 
-	if (IS_STICKY(fw))
+	return 1;
+}
+
+int stick_across_desks(F_CMD_ARGS, int toggle)
+{
+	FvwmWindow *fw = exc->w.fw;
+
+	if ((toggle == 1 && IS_STICKY_ACROSS_DESKS(fw)) ||
+	    (toggle == 0 && !IS_STICKY_ACROSS_DESKS(fw)))
 	{
-		SET_STICKY(fw, 0);
+		return 0;
+	}
+
+	if (IS_STICKY_ACROSS_DESKS(fw))
+	{
+		SET_STICKY_ACROSS_DESKS(fw, 0);
 		fw->Desk = Scr.CurrentDesk;
 		GNOME_SetDeskCount();
 		GNOME_SetDesk(fw);
@@ -4027,16 +4133,15 @@ void handle_stick(F_CMD_ARGS, int toggle, int do_not_draw, int do_silently)
 		{
 			do_move_window_to_desk(fw, Scr.CurrentDesk);
 		}
-		SET_STICKY(fw, 1);
-		if (!IsRectangleOnThisPage(&fw->frame_g, Scr.CurrentDesk))
-		{
-			action = "";
-			__move_window(F_PASS_ARGS, FALSE, MOVE_PAGE);
-			/* __move_window resets the STICKY flag, so we must
-			 * set it after the call! */
-			SET_STICKY(fw, 1);
-		}
+		SET_STICKY_ACROSS_DESKS(fw, 1);
 	}
+
+	return 1;
+}
+
+static void __handle_stick_exit(
+	FvwmWindow *fw, int do_not_draw, int do_silently)
+{
 	if (do_not_draw == 0)
 	{
 		border_draw_decorations(
@@ -4054,12 +4159,87 @@ void handle_stick(F_CMD_ARGS, int toggle, int do_not_draw, int do_silently)
 	return;
 }
 
+void handle_stick_across_pages(
+	F_CMD_ARGS, int toggle, int do_not_draw, int do_silently)
+{
+	FvwmWindow *fw = exc->w.fw;
+	int did_change;
+
+	did_change = stick_across_pages(F_PASS_ARGS, toggle);
+	if (did_change)
+	{
+		__handle_stick_exit(fw, do_not_draw, do_silently);
+	}
+
+	return;
+}
+
+void handle_stick_across_desks(
+	F_CMD_ARGS, int toggle, int do_not_draw, int do_silently)
+{
+	FvwmWindow *fw = exc->w.fw;
+	int did_change;
+
+	did_change = stick_across_desks(F_PASS_ARGS, toggle);
+	if (did_change)
+	{
+		__handle_stick_exit(fw, do_not_draw, do_silently);
+	}
+
+	return;
+}
+
+void handle_stick(
+	F_CMD_ARGS, int toggle_page, int toggle_desk, int do_not_draw,
+	int do_silently)
+{
+	FvwmWindow *fw = exc->w.fw;
+	int did_change;
+
+	did_change = 0;
+	did_change |= stick_across_desks(F_PASS_ARGS, toggle_desk);
+	did_change |= stick_across_pages(F_PASS_ARGS, toggle_page);
+	if (did_change)
+	{
+		__handle_stick_exit(fw, do_not_draw, do_silently);
+	}
+
+	return;
+}
+
 void CMD_Stick(F_CMD_ARGS)
 {
 	int toggle;
 
 	toggle = ParseToggleArgument(action, &action, -1, 0);
-	handle_stick(F_PASS_ARGS, toggle, 0, 0);
+	if (toggle == -1 && IS_STICKY_ACROSS_DESKS(exc->w.fw) !=
+	    IS_STICKY_ACROSS_PAGES(exc->w.fw))
+	{
+		/* don't switch between only stickypage and only stickydesk.
+		 * rather switch it off completely */
+		toggle = 0;
+	}
+	handle_stick(F_PASS_ARGS, toggle, toggle, 0, 0);
+
+	return;
+}
+
+void CMD_StickAcrossPages(F_CMD_ARGS)
+{
+	int toggle;
+
+	toggle = ParseToggleArgument(action, &action, -1, 0);
+	handle_stick_across_pages(F_PASS_ARGS, toggle, 0, 0);
+
+	return;
+}
+
+void CMD_StickAcrossDesks(F_CMD_ARGS)
+{
+	int toggle;
+
+	toggle = ParseToggleArgument(action, &action, -1, 0);
+	handle_stick_across_desks(F_PASS_ARGS, toggle, 0, 0);
 
 	return;
 }
