@@ -14,7 +14,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-/* ---------------------------- included header files ----------------------- */
+/* ---------------------------- included header files ---------------------- */
 
 #include "config.h"
 
@@ -22,6 +22,7 @@
 
 #include "libs/fvwmlib.h"
 #include "libs/FScreen.h"
+#include "libs/FGettext.h"
 #include "fvwm.h"
 #include "externs.h"
 #include "execcontext.h"
@@ -42,19 +43,19 @@
 #include "icons.h"
 #include "stack.h"
 
-/* ---------------------------- local definitions --------------------------- */
+/* ---------------------------- local definitions -------------------------- */
 
-/* ---------------------------- local macros -------------------------------- */
+/* ---------------------------- local macros ------------------------------- */
 
-/* ---------------------------- imports ------------------------------------- */
+/* ---------------------------- imports ------------------------------------ */
 
-/* ---------------------------- included code files ------------------------- */
+/* ---------------------------- included code files ------------------------ */
 
-/* ---------------------------- local types --------------------------------- */
+/* ---------------------------- local types -------------------------------- */
 
-/* ---------------------------- forward declarations ------------------------ */
+/* ---------------------------- forward declarations ----------------------- */
 
-/* ---------------------------- local variables ----------------------------- */
+/* ---------------------------- local variables ---------------------------- */
 
 /*
  * dje 12/19/98
@@ -84,11 +85,11 @@ static int prev_desk_and_page_desk = 0;
 static unsigned int prev_desk_and_page_page_x = 0;
 static unsigned int prev_desk_and_page_page_y = 0;
 
-/* ---------------------------- exported variables (globals) ---------------- */
+/* ---------------------------- exported variables (globals) --------------- */
 
-/* ---------------------------- local functions ----------------------------- */
+/* ---------------------------- local functions ---------------------------- */
 
-/**************************************************************************
+/*
  *
  * Parse arguments for "Desk" and "MoveToDesk" (formerly "WindowsDesk"):
  *
@@ -105,7 +106,7 @@ static unsigned int prev_desk_and_page_page_y = 0;
  * The current desk number is returned if not enough parameters could be
  * read (or if action is empty).
  *
- **************************************************************************/
+ */
 static int GetDeskNumber(char *action)
 {
 	int n;
@@ -196,11 +197,11 @@ static int GetDeskNumber(char *action)
 	return desk;
 }
 
-/**************************************************************************
+/*
  *
  * Unmaps a window on transition to a new desktop
  *
- *************************************************************************/
+ */
 static void unmap_window(FvwmWindow *t)
 {
 	XWindowAttributes winattrs;
@@ -250,11 +251,11 @@ static void unmap_window(FvwmWindow *t)
 	return;
 }
 
-/**************************************************************************
+/*
  *
  * Maps a window on transition to a new desktop
  *
- *************************************************************************/
+ */
 static void map_window(FvwmWindow *t)
 {
 	XWindowAttributes winattrs;
@@ -312,14 +313,14 @@ static void map_window(FvwmWindow *t)
 	return;
 }
 
-/************************************************************************
+/*
  *
  * Unmap all windows on a desk -
  *   - Part 1 of a desktop switch
  *   - must eventually be followed by a call to MapDesk
  *   - unmaps from the bottom of the stack up
  *
- *************************************************************************/
+ */
 static void UnmapDesk(int desk, Bool grab)
 {
 	FvwmWindow *t;
@@ -333,8 +334,7 @@ static void UnmapDesk(int desk, Bool grab)
 	     t != &Scr.FvwmRoot; t = get_prev_window_in_stack_ring(t))
 	{
 		/* Only change mapping for non-sticky windows */
-		if (!(IS_ICONIFIED(t) && IS_ICON_STICKY(t)) &&
-		   !(IS_STICKY(t)) && !IS_ICON_UNMAPPED(t))
+		if (!is_window_sticky_across_desks(t) && !IS_ICON_UNMAPPED(t))
 		{
 			if (t->Desk == desk)
 			{
@@ -349,6 +349,8 @@ static void UnmapDesk(int desk, Bool grab)
 					t->flags.is_focused_on_other_desk = 0;
 				}
 				unmap_window(t);
+				SET_FULLY_VISIBLE(t, 0);
+				SET_PARTIALLY_VISIBLE(t, 0);
 			}
 		}
 		else
@@ -364,14 +366,14 @@ static void UnmapDesk(int desk, Bool grab)
 	return;
 }
 
-/************************************************************************
+/*
  *
  * Map all windows on a desk -
  *   - Part 2 of a desktop switch
  *   - only use if UnmapDesk has previously been called
  *   - maps from the top of the stack down
  *
- ************************************************************************/
+ */
 static void MapDesk(int desk, Bool grab)
 {
 	FvwmWindow *t;
@@ -388,8 +390,7 @@ static void MapDesk(int desk, Bool grab)
 	     t != &Scr.FvwmRoot; t = get_next_window_in_stack_ring(t))
 	{
 		/* Only change mapping for non-sticky windows */
-		if (!(IS_ICONIFIED(t) && IS_ICON_STICKY(t)) &&
-		   !(IS_STICKY(t)) && !IS_ICON_UNMAPPED(t))
+		if (!is_window_sticky_across_desks(t) && !IS_ICON_UNMAPPED(t))
 		{
 			if (t->Desk == desk)
 			{
@@ -401,6 +402,8 @@ static void MapDesk(int desk, Bool grab)
 			/*  If window is sticky, just update its desk (it's
 			 * still mapped).  */
 			t->Desk = desk;
+			GNOME_SetDesk(t);
+			GNOME_SetWinArea(t);
 			if (sf == t)
 			{
 				StickyWin = t;
@@ -418,9 +421,8 @@ static void MapDesk(int desk, Bool grab)
 		  Autoplace any sticky icons, so that sticky icons from the old
 		  desk don't land on top of stationary ones on the new desk.
 		*/
-		if ((IS_STICKY(t) || IS_ICON_STICKY(t)) &&
-		   IS_ICONIFIED(t) && !IS_ICON_MOVED(t) &&
-		   !IS_ICON_UNMAPPED(t))
+		if (is_window_sticky_across_desks(t) && IS_ICONIFIED(t) &&
+		    !IS_ICON_MOVED(t) && !IS_ICON_UNMAPPED(t))
 		{
 			AutoPlaceIcon(t, NULL, True);
 		}
@@ -451,9 +453,9 @@ static void MapDesk(int desk, Bool grab)
 	return;
 }
 
-/* ---------------------------- interface functions ------------------------- */
+/* ---------------------------- interface functions ------------------------ */
 
-/***************************************************************************
+/*
  *
  * Check to see if the pointer is on the edge of the screen, and scroll/page
  * if needed
@@ -462,7 +464,7 @@ static void MapDesk(int desk, Bool grab)
  *  0: no paging
  *  1: paging occured
  * -1: no need to call the function again before a new event arrives
- ***************************************************************************/
+ */
 int HandlePaging(
 	XEvent *pev, int HorWarpSize, int VertWarpSize, int *xl, int *yt,
 	int *delta_x, int *delta_y, Bool Grab, Bool fLoop,
@@ -765,11 +767,11 @@ int HandlePaging(
  * Hermann Dunkel, HEDU, dunkel@cul-ipn.uni-kiel.de 1/94
  */
 
-/***************************************************************************
+/*
  * checkPanFrames hides PanFrames if they are on the very border of the
  * VIRTUAL screen and EdgeWrap for that direction is off.
  * (A special cursor for the EdgeWrap border could be nice) HEDU
- ****************************************************************************/
+ */
 void checkPanFrames(void)
 {
 	Bool do_unmap_l = False;
@@ -941,14 +943,14 @@ void checkPanFrames(void)
 	return;
 }
 
-/****************************************************************************
+/*
  *
  * Gotta make sure these things are on top of everything else, or they
  * don't work!
  *
  * For some reason, this seems to be unneeded.
  *
- ***************************************************************************/
+ */
 void raisePanFrames(void)
 {
 	if (Scr.PanFrameTop.isMapped)
@@ -971,11 +973,11 @@ void raisePanFrames(void)
 	return;
 }
 
-/****************************************************************************
+/*
  *
  * Creates the windows for edge-scrolling
  *
- ****************************************************************************/
+ */
 void initPanFrames(void)
 {
 	XSetWindowAttributes attributes;
@@ -1037,11 +1039,11 @@ Bool is_pan_frame(Window w)
 	}
 }
 
-/***************************************************************************
+/*
  *
  *  Moves the viewport within the virtual desktop
  *
- ***************************************************************************/
+ */
 void MoveViewport(int newx, int newy, Bool grab)
 {
 	FvwmWindow *t, *t1;
@@ -1063,11 +1065,11 @@ void MoveViewport(int newx, int newy, Bool grab)
 	{
 		newy = Scr.VyMax;
 	}
-	if (newx <0)
+	if (newx < 0)
 	{
 		newx = 0;
 	}
-	if (newy <0)
+	if (newy < 0)
 	{
 		newy = 0;
 	}
@@ -1095,8 +1097,10 @@ void MoveViewport(int newx, int newy, Bool grab)
 	if (deltax || deltay)
 	{
 		BroadcastPacket(
-			M_NEW_PAGE, 5, Scr.Vx, Scr.Vy, Scr.CurrentDesk,
-			Scr.VxMax, Scr.VyMax);
+			M_NEW_PAGE, 7, Scr.Vx, Scr.Vy, Scr.CurrentDesk,
+			Scr.MyDisplayWidth, Scr.MyDisplayHeight,
+			(int)(Scr.VxMax / Scr.MyDisplayWidth) + 1,
+			(int)(Scr.VyMax / Scr.MyDisplayHeight) + 1);
 
 		/*
 		 * RBW - 11/13/1998      - new:  chase the chain
@@ -1121,8 +1125,7 @@ void MoveViewport(int newx, int newy, Bool grab)
 			tyt = t->frame_g.y;
 			txr = t->frame_g.x + t->frame_g.width - 1;
 			tyb = t->frame_g.y + t->frame_g.height - 1;
-			if ((IS_STICKY(t) ||
-			     (IS_ICONIFIED(t) && IS_ICON_STICKY(t))) &&
+			if (is_window_sticky_across_pages(t) &&
 			    !IS_VIEWPORT_MOVED(t))
 			{
 				/* the absolute position has changed */
@@ -1143,8 +1146,7 @@ void MoveViewport(int newx, int newy, Bool grab)
 				/* If the window is iconified, and sticky
 				 * Icons is set, then the window should
 				 * essentially be sticky */
-				if (!(IS_ICONIFIED(t) && IS_ICON_STICKY(t)) &&
-				    !IS_STICKY(t))
+				if (!is_window_sticky_across_pages(t))
 				{
 					if (IS_ICONIFIED(t))
 					{
@@ -1183,8 +1185,7 @@ void MoveViewport(int newx, int newy, Bool grab)
 				/* If the window is iconified, and sticky
 				 * Icons is set, then the window should
 				 * essentially be sticky */
-				if (!(IS_ICONIFIED(t1) &&
-				      IS_ICON_STICKY(t1)) && !IS_STICKY(t1))
+				if (!is_window_sticky_across_pages(t1))
 				{
 					if (IS_ICONIFIED(t1))
 					{
@@ -1215,9 +1216,9 @@ void MoveViewport(int newx, int newy, Bool grab)
 			/* If its an icon, and its sticking, autoplace it so
 			 * that it doesn't wind up on top a a stationary
 			 * icon */
-			if ((IS_STICKY(t) || IS_ICON_STICKY(t)) &&
-			   IS_ICONIFIED(t) && !IS_ICON_MOVED(t) &&
-			   !IS_ICON_UNMAPPED(t))
+			if (is_window_sticky_across_pages(t) &&
+			    IS_ICONIFIED(t) && !IS_ICON_MOVED(t) &&
+			    !IS_ICON_UNMAPPED(t))
 			{
 				AutoPlaceIcon(t, NULL, True);
 			}
@@ -1238,6 +1239,7 @@ void MoveViewport(int newx, int newy, Bool grab)
 	}
 	/* update GNOME pager */
 	GNOME_SetCurrentArea();
+	EWMH_SetDesktopViewPort();
 
 	return;
 }
@@ -1276,11 +1278,11 @@ void goto_desk(int desk)
 	return;
 }
 
-/**************************************************************************
+/*
  *
  * Move a window to a new desktop
  *
- *************************************************************************/
+ */
 void do_move_window_to_desk(FvwmWindow *fw, int desk)
 {
 	if (fw == NULL)
@@ -1292,8 +1294,7 @@ void do_move_window_to_desk(FvwmWindow *fw, int desk)
 	 * Set the window's desktop, and map or unmap it as needed.
 	 */
 	/* Only change mapping for non-sticky windows */
-	if (!(IS_ICONIFIED(fw) && IS_ICON_STICKY(fw)) &&
-	    !IS_STICKY(fw) /*&& !IS_ICON_UNMAPPED(fw)*/)
+	if (!is_window_sticky_across_desks(fw) /*&& !IS_ICON_UNMAPPED(fw)*/)
 	{
 		if (fw->Desk == Scr.CurrentDesk)
 		{
@@ -1303,6 +1304,8 @@ void do_move_window_to_desk(FvwmWindow *fw, int desk)
 				DeleteFocus(True);
 			}
 			unmap_window(fw);
+			SET_FULLY_VISIBLE(fw, 0);
+			SET_PARTIALLY_VISIBLE(fw, 0);
 		}
 		else if (desk == Scr.CurrentDesk)
 		{
@@ -1380,7 +1383,8 @@ Bool get_page_arguments(char *action, int *page_x, int *page_y)
 		free(token);
 	}
 
-	if (GetSuffixedIntegerArguments(action, NULL, val, 2, "p", suffix) != 2)
+	if (GetSuffixedIntegerArguments(action, NULL, val, 2, "p", suffix) !=
+	    2)
 	{
 		return 0;
 	}
@@ -1449,12 +1453,12 @@ char *GetDesktopName(int desk)
 	return NULL;
 }
 
-/* ---------------------------- builtin commands ---------------------------- */
+/* ---------------------------- builtin commands --------------------------- */
 
 /* EdgeCommand - binds a function to a pan frame enter event */
 void CMD_EdgeCommand(F_CMD_ARGS)
 {
-	direction_type direction;
+	direction_t direction;
 	char * command;
 
 	/* get the direction */
@@ -1584,7 +1588,7 @@ void CMD_EdgeThickness(F_CMD_ARGS)
 
 void CMD_EdgeScroll(F_CMD_ARGS)
 {
-	int val1, val2, val1_unit,val2_unit,n;
+	int val1, val2, val1_unit, val2_unit, n;
 
 	n = GetTwoArguments(action, &val1, &val2, &val1_unit, &val2_unit);
 	if (n != 2)
@@ -1596,7 +1600,7 @@ void CMD_EdgeScroll(F_CMD_ARGS)
 	}
 
 	/*
-	 * if edgescroll >1000 and < 100000m
+	 * if edgescroll >1000 and <100000
 	 * wrap at edges of desktop (a "spherical" desktop)
 	 */
 	if (val1 >= 1000)
@@ -1618,8 +1622,8 @@ void CMD_EdgeScroll(F_CMD_ARGS)
 		Scr.flags.edge_wrap_y = 0;
 	}
 
-	Scr.EdgeScrollX = val1*val1_unit/100;
-	Scr.EdgeScrollY = val2*val2_unit/100;
+	Scr.EdgeScrollX = val1 * val1_unit / 100;
+	Scr.EdgeScrollY = val2 * val2_unit / 100;
 
 	checkPanFrames();
 
@@ -1760,21 +1764,25 @@ void CMD_DesktopSize(F_CMD_ARGS)
 		0: val[0]*Scr.MyDisplayWidth-Scr.MyDisplayWidth;
 	Scr.VyMax = (val[1] <= 0) ?
 		0: val[1]*Scr.MyDisplayHeight-Scr.MyDisplayHeight;
-	BroadcastPacket(M_NEW_PAGE, 5,
-			Scr.Vx, Scr.Vy, Scr.CurrentDesk, Scr.VxMax, Scr.VyMax);
+	BroadcastPacket(
+		M_NEW_PAGE, 7, Scr.Vx, Scr.Vy, Scr.CurrentDesk,
+		Scr.MyDisplayWidth, Scr.MyDisplayHeight,
+		(int)(Scr.VxMax / Scr.MyDisplayWidth) + 1,
+		(int)(Scr.VyMax / Scr.MyDisplayHeight) + 1);
 
 	checkPanFrames();
 	/* update GNOME pager */
 	GNOME_SetAreaCount();
+	EWMH_SetDesktopGeometry();
 
 	return;
 }
 
-/**************************************************************************
+/*
  *
  * Move to a new desktop
  *
- *************************************************************************/
+ */
 void CMD_GotoDesk(F_CMD_ARGS)
 {
 	goto_desk(GetDeskNumber(action));
@@ -1789,7 +1797,7 @@ void CMD_Desk(F_CMD_ARGS)
 	return;
 }
 
-/*************************************************************************
+/*
  *
  * Move to a new desktop and page at the same time.
  *   This function is designed for use by the Pager, and replaces the old
@@ -1797,7 +1805,7 @@ void CMD_Desk(F_CMD_ARGS)
  *   - unmap all windows on the current desk so they don't flash when the
  *     viewport is moved, then switch the viewport, then the desk.
  *
- *************************************************************************/
+ */
 void CMD_GotoDeskAndPage(F_CMD_ARGS)
 {
 	int val[3];
@@ -1809,7 +1817,12 @@ void CMD_GotoDeskAndPage(F_CMD_ARGS)
 		val[1] = prev_desk_and_page_page_x;
 		val[2] = prev_desk_and_page_page_y;
 	}
-	else if (GetIntegerArguments(action, NULL, val, 3) != 3)
+	else if (GetIntegerArguments(action, NULL, val, 3) == 3)
+	{
+		val[1] *= Scr.MyDisplayWidth;
+		val[2] *= Scr.MyDisplayHeight;
+	}
+	else
 	{
 		return;
 	}
@@ -1821,9 +1834,7 @@ void CMD_GotoDeskAndPage(F_CMD_ARGS)
 	}
 	prev_desk_and_page_page_x = Scr.Vx;
 	prev_desk_and_page_page_y = Scr.Vy;
-	MoveViewport(
-		val[1] * Scr.MyDisplayWidth, val[2] * Scr.MyDisplayHeight,
-		True);
+	MoveViewport(val[1], val[2], True);
 	if (is_new_desk)
 	{
 		prev_desk = Scr.CurrentDesk;
@@ -1898,7 +1909,6 @@ void CMD_Scroll(F_CMD_ARGS)
 		/* to few parameters */
 		return;
 	}
-
 	if ((val1 > -100000)&&(val1 < 100000))
 	{
 		x = Scr.Vx + val1*val1_unit/100;
@@ -1907,7 +1917,6 @@ void CMD_Scroll(F_CMD_ARGS)
 	{
 		x = Scr.Vx + (val1/1000)*val1_unit/100;
 	}
-
 	if ((val2 > -100000)&&(val2 < 100000))
 	{
 		y=Scr.Vy + val2*val2_unit/100;
@@ -1916,7 +1925,6 @@ void CMD_Scroll(F_CMD_ARGS)
 	{
 		y = Scr.Vy + (val2/1000)*val2_unit/100;
 	}
-
 	if (((val1 <= -100000)||(val1 >= 100000))&&(x>Scr.VxMax))
 	{
 		int xpixels;
@@ -1966,11 +1974,11 @@ void CMD_Scroll(F_CMD_ARGS)
 	return;
 }
 
-/**************************************************************************
+/*
  *
  * Defines the name of a desktop
  *
- *************************************************************************/
+ */
 void CMD_DesktopName(F_CMD_ARGS)
 {
 	int desk;
@@ -2046,6 +2054,8 @@ void CMD_DesktopName(F_CMD_ARGS)
 	if (!fFvwmInStartup)
 	{
 		char *msg;
+		const char *default_desk_name = _("Desk");
+
 		/* should send the info to the FvwmPager and set the EWMH
 		 * desktop names */
 		if (action != NULL && *action && *action != '\n')
@@ -2055,8 +2065,10 @@ void CMD_DesktopName(F_CMD_ARGS)
 		}
 		else
 		{
-			msg = (char *)safemalloc(84);
-			sprintf(msg, "DesktopName %d Desk %d", desk, desk);
+			msg = (char *)safemalloc(strlen(default_desk_name)+44);
+			sprintf(
+				msg, "DesktopName %d %s %d", desk,
+				default_desk_name, desk);
 		}
 		BroadcastConfigInfoString(msg);
 		free(msg);
