@@ -1,7 +1,8 @@
-/****************************************************************************
+/* -*-c-*- */
+/*
  * This module is all new
  * by Rob Nation
- ****************************************************************************/
+ */
 
 /* This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,11 +19,11 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307	 USA
  */
 
-/***********************************************************************
+/*
  *
  * fvwm pager handling code
  *
- ***********************************************************************/
+ */
 
 #include "config.h"
 
@@ -154,7 +155,7 @@ static void discard_events(long event_type, Window w, XEvent *last_ev)
   return;
 }
 
-/***********************************************************************
+/*
  *
  *  Procedure:
  *	CalcGeom - calculates the size and position of a mini-window
@@ -162,7 +163,7 @@ static void discard_events(long event_type, Window w, XEvent *last_ev)
  *	You can always tell bad code by the size of the comments.
  *
  *	dv: Some people say that good code does not nee lengthy comments. :-)
- ***********************************************************************/
+ */
 static void CalcGeom(PagerWindow *t, int win_w, int win_h,
 		     int *x_ret, int *y_ret, int *w_ret, int *h_ret)
 {
@@ -245,12 +246,12 @@ static void CalcGeom(PagerWindow *t, int win_w, int win_h,
   *h_ret = size;
 }
 
-/***********************************************************************
+/*
  *
  *  Procedure:
  *	Initialize_viz_pager - creates a temp window of the correct visual
  *	so that pixmaps may be created for use with the main window
- ***********************************************************************/
+ */
 void initialize_viz_pager(void)
 {
 	XSetWindowAttributes attr;
@@ -374,7 +375,7 @@ void draw_desk_background(int i, int page_w, int page_h)
 	return;
 }
 
-/***********************************************************************
+/*
  *
  *  Procedure:
  *	Initialize_pager - creates the pager window, if needed
@@ -382,7 +383,7 @@ void draw_desk_background(int i, int page_w, int page_h)
  *  Inputs:
  *	x,y location of the window
  *
- ***********************************************************************/
+ */
 char *pager_name = "Fvwm Pager";
 XSizeHints sizehints =
 {
@@ -440,6 +441,7 @@ void initialize_pager(void)
   int n,m,w,h,i,x,y;
   XGCValues gcv;
   char dash_list[2];
+  FlocaleFont *balloon_font;
 
   /* I don't think that this is necessary - just let pager die */
   /* domivogt (07-mar-1999): But it is! A window being moved in the pager
@@ -619,8 +621,6 @@ void initialize_pager(void)
   }
   sizehints.width = window_w;
   sizehints.height = window_h;
-  sizehints.x = window_x;
-  sizehints.y = window_y;
 
   if(usposition)
     sizehints.flags |= USPosition;
@@ -638,6 +638,8 @@ void initialize_pager(void)
 			       window_h, 0, Pdepth, InputOutput, Pvisual,
 			       valuemask, &attributes);
   XSetWMProtocols(dpy,Scr.Pager_w,&wm_del_win,1);
+  /* hack to prevent mapping on wrong screen with StartsOnScreen */
+  FScreenMangleScreenIntoUSPosHints(FSCREEN_XYPOS, &sizehints);
   XSetWMNormalHints(dpy,Scr.Pager_w,&sizehints);
   if (is_transient)
   {
@@ -742,6 +744,7 @@ void initialize_pager(void)
     Scr.asGC = fvwmlib_XCreateGC(dpy, Scr.Pager_w, GCForeground, &gcv);
   }
 
+  balloon_font = FlocaleLoadFont(dpy, BalloonFont, MyName);
   for(i=0;i<ndesks;i++)
   {
     w = window_w / Columns;
@@ -902,7 +905,7 @@ void initialize_pager(void)
     XMapRaised(dpy,Desks[i].title_w);
 
     /* get font for balloon */
-    Desks[i].balloon.Ffont = FlocaleLoadFont(dpy, BalloonFont, MyName);
+    Desks[i].balloon.Ffont = balloon_font;
     if (Desks[i].balloon.Ffont == NULL)
     {
       fprintf(stderr, "%s: No fonts available, giving up!.\n", MyName);
@@ -969,11 +972,11 @@ void UpdateWindowShape ()
 
 
 
-/****************************************************************************
+/*
  *
  * Decide what to do about received X events
  *
- ****************************************************************************/
+ */
 void DispatchEvent(XEvent *Event)
 {
   int i,x,y;
@@ -1293,11 +1296,11 @@ void HandleExpose(XEvent *Event)
 	discard_events(Expose, Event->xany.window, NULL);
 }
 
-/****************************************************************************
+/*
  *
  * Respond to a change in window geometry.
  *
- ****************************************************************************/
+ */
 void ReConfigure(void)
 {
   Window root;
@@ -1352,7 +1355,6 @@ void ReConfigure(void)
     for(j=0;j<Columns;j++)
     {
       i = k*Columns+j;
-
       if (i<ndesks)
       {
 	XMoveResizeWindow(
@@ -1366,7 +1368,6 @@ void ReConfigure(void)
 		{
 			draw_desk_background(i, w, h);
 		}
-		continue;
 	}
 	if (HilightDesks)
 	{
@@ -1383,11 +1384,11 @@ void ReConfigure(void)
   ReConfigureAll();
 }
 
-/****************************************************************************
+/*
  *
  * Respond to a "background" change: update the Parental Relative cset
  *
- ****************************************************************************/
+ */
 
 /* layout:
  * Root -> pager window (pr) -> title window -> desk window -> window view
@@ -1614,11 +1615,11 @@ void ReConfigureIcons(Bool do_reconfigure_desk_only)
   }
 }
 
-/****************************************************************************
+/*
  *
  * Draw grid lines for desk #i
  *
- ****************************************************************************/
+ */
 void DrawGrid(int desk, int erase, Window ew, XRectangle *r)
 {
 	int y, y1, y2, x, x1, x2,d,w;
@@ -1912,79 +1913,95 @@ void IconSwitchPage(XEvent *Event)
 
 void AddNewWindow(PagerWindow *t)
 {
-  unsigned long valuemask;
-  XSetWindowAttributes attributes;
-  int i, x, y, w, h;
+	unsigned long valuemask;
+	XSetWindowAttributes attributes;
+	int i, x, y, w, h;
 
-  i = t->desk - desk1;
-  CalcGeom(t, desk_w, desk_h, &x, &y, &w, &h);
-  t->pager_view_x = x;
-  t->pager_view_y = y;
-  t->pager_view_width = w;
-  t->pager_view_height = h;
-  valuemask = CWBackPixel | CWEventMask;
-  attributes.background_pixel = t->back;
-  attributes.event_mask = ExposureMask;
+	i = t->desk - desk1;
+	CalcGeom(t, desk_w, desk_h, &x, &y, &w, &h);
+	t->pager_view_x = x;
+	t->pager_view_y = y;
+	t->pager_view_width = w;
+	t->pager_view_height = h;
+	valuemask = CWBackPixel | CWEventMask;
+	attributes.background_pixel = t->back;
+	attributes.event_mask = ExposureMask;
 
-  /* ric@giccs.georgetown.edu -- added Enter and Leave events for
-     popping up balloon window */
-  attributes.event_mask = ExposureMask | EnterWindowMask | LeaveWindowMask;
+	/* ric@giccs.georgetown.edu -- added Enter and Leave events for
+	   popping up balloon window */
+	attributes.event_mask =
+		ExposureMask | EnterWindowMask | LeaveWindowMask;
 
-  if ((i >= 0) && (i < ndesks))
-  {
-    t->PagerView = XCreateWindow(dpy,Desks[i].w, x, y, w, h, 0,
-				 CopyFromParent, InputOutput, CopyFromParent,
-				 valuemask, &attributes);
-    if (windowcolorset > -1)
-    {
-      SetWindowBackground(
-	dpy, t->PagerView, w, h, &Colorset[windowcolorset], Pdepth,
-	Scr.NormalGC, True);
-    }
-    if (!UseSkipList || !DO_SKIP_WINDOW_LIST(t))
-    {
-      XMapRaised(dpy, t->PagerView);
-    }
-  }
-  else
-  {
-    t->PagerView = None;
-  }
+	if ((i >= 0) && (i < ndesks))
+	{
+		t->PagerView = XCreateWindow(
+			dpy,Desks[i].w, x, y, w, h, 0, CopyFromParent,
+			InputOutput, CopyFromParent, valuemask, &attributes);
+		if (windowcolorset > -1)
+		{
+			SetWindowBackground(
+				dpy, t->PagerView, w, h,
+				&Colorset[windowcolorset], Pdepth,
+				Scr.NormalGC, True);
+		}
+		if (!UseSkipList || !DO_SKIP_WINDOW_LIST(t))
+		{
+			if (IS_ICONIFIED(t))
+			{
+				XMoveResizeWindow(
+					dpy, t->PagerView, -32768, -32768, 1,
+					1);
+			}
+			XMapRaised(dpy, t->PagerView);
+		}
+	}
+	else
+	{
+		t->PagerView = None;
+	}
 
-  CalcGeom(t, icon_w, icon_h, &x, &y, &w, &h);
-  t->icon_view_width = w;
-  t->icon_view_height = h;
-  if(Scr.CurrentDesk != t->desk)
-  {
-    x = -32768;
-    y = -32768;
-  }
-  t->IconView = XCreateWindow(
-    dpy,icon_win, x, y, w, h, 0, CopyFromParent, InputOutput,
-    CopyFromParent, valuemask, &attributes);
-  if (windowcolorset > -1)
-  {
-    SetWindowBackground(
-      dpy, t->IconView, w, h, &Colorset[windowcolorset], Pdepth,
-      Scr.NormalGC, True);
-  }
-  if(Scr.CurrentDesk == t->desk)
-  {
-    XGrabButton(
-      dpy, 2, AnyModifier, t->IconView, True,
-      ButtonPressMask | ButtonReleaseMask|ButtonMotionMask,
-      GrabModeAsync, GrabModeAsync, None, None);
-  }
-  if (!UseSkipList || !DO_SKIP_WINDOW_LIST(t))
-  {
-    XMapRaised(dpy,t->IconView);
-    t->myflags.is_mapped = 1;
-  }
-  else
-  {
-    t->myflags.is_mapped = 0;
-  }
-  Hilight(t,False);
+	CalcGeom(t, icon_w, icon_h, &x, &y, &w, &h);
+	t->icon_view_width = w;
+	t->icon_view_height = h;
+	if(Scr.CurrentDesk != t->desk)
+	{
+		x = -32768;
+		y = -32768;
+	}
+	t->IconView = XCreateWindow(
+		dpy,icon_win, x, y, w, h, 0, CopyFromParent, InputOutput,
+		CopyFromParent, valuemask, &attributes);
+	if (windowcolorset > -1)
+	{
+		SetWindowBackground(
+			dpy, t->IconView, w, h, &Colorset[windowcolorset],
+			Pdepth, Scr.NormalGC, True);
+	}
+	if(Scr.CurrentDesk == t->desk)
+	{
+		XGrabButton(
+			dpy, 2, AnyModifier, t->IconView, True,
+			ButtonPressMask | ButtonReleaseMask|ButtonMotionMask,
+			GrabModeAsync, GrabModeAsync, None, None);
+	}
+	if (!UseSkipList || !DO_SKIP_WINDOW_LIST(t))
+	{
+		if (IS_ICONIFIED(t))
+		{
+			XMoveResizeWindow(
+				dpy, t->IconView, -32768, -32768, 1,
+				1);
+		}
+		XMapRaised(dpy, t->IconView);
+		t->myflags.is_mapped = 1;
+	}
+	else
+	{
+		t->myflags.is_mapped = 0;
+	}
+	Hilight(t,False);
+
+	return;
 }
 
 
@@ -2236,7 +2253,7 @@ void Scroll(int window_w, int window_h, int x, int y, int Desk,
 	int adjx,adjy;
 
 	/* Desk < 0 means we want to scroll an icon window */
-	if(Desk >= 0 && Desk + desk1 != Scr.CurrentDesk)
+	if (Desk >= 0 && Desk + desk1 != Scr.CurrentDesk)
 	{
 		return;
 	}
@@ -2248,39 +2265,39 @@ void Scroll(int window_w, int window_h, int x, int y, int Desk,
 	y-=adjy/2;
 
 	/* adjust for pointer going out of range */
-	if(x < 0)
+	if (x < 0)
 	{
 		x = 0;
 	}
-	if(y < 0)
+	if (y < 0)
 	{
 		y = 0;
 	}
-	
-	if(x > window_w-adjx)
+
+	if (x > window_w-adjx)
 	{
 		x = window_w-adjx;
 	}
-	if(y > window_h-adjy)
+	if (y > window_h-adjy)
 	{
 		y = window_h-adjy;
 	}
 
 	sx = 0;
 	sy = 0;
-	if(window_w != 0)
+	if (window_w != 0)
 	{
 		sx = 100 * (x * Scr.VWidth / window_w - MyVx) /
 			Scr.MyDisplayWidth;
 	}
-	if(window_h != 0)
+	if (window_h != 0)
 	{
 		sy = 100 * (y * Scr.VHeight / window_h - MyVy) /
 			Scr.MyDisplayHeight;
 	}
 	MYFPRINTF((stderr,"[scroll]: %d %d %d %d %d %d\n", window_w, window_h, x,
 		y, sx, sy));
-	/* Make sure weExecuteCommandQueue(); don't get stuck a few pixels fromt
+	/* Make sure weExecuteCommandQueue(); don't get stuck a few pixels from
 	 * the top/left border. Since sx/sy are ints, values between 0 and 1 are
 	 * rounded down. */
 	if (sx == 0 && x == 0 && MyVx != 0)
@@ -2291,7 +2308,10 @@ void Scroll(int window_w, int window_h, int x, int y, int Desk,
 	{
 		sy = -1;
 	}
-	if (sx==0&&sy==0) return;
+	if (sx == 0 && sy == 0)
+	{
+		return;
+	}
 	if (Wait == 0 || last_sx != sx || last_sy != sy)
 	{
 		sprintf(command, "Scroll %d %d", sx, sy);
@@ -2299,8 +2319,16 @@ void Scroll(int window_w, int window_h, int x, int y, int Desk,
 		/* Here we need to track the view offset on the desk. */
 		/* sx/y are are a percent of the screen to scroll. */
 		/* We don't use Scr.Vx/y since they lag the true position. */
-		MyVx+=Scr.MyDisplayWidth*sx/100;
-		MyVy+=Scr.MyDisplayHeight*sy/100;
+		MyVx += Scr.MyDisplayWidth * sx / 100;
+		if (MyVx < 0)
+		{
+			MyVx = 0;
+		}
+		MyVy += Scr.MyDisplayHeight * sy / 100;
+		if (MyVy < 0)
+		{
+			MyVy = 0;
+		}
 		Wait = 1;
 	}
 	if (Wait == 0)
@@ -2581,12 +2609,12 @@ void MoveWindow(XEvent *Event)
 }
 
 
-/***********************************************************************
+/*
  *
  *  Procedure:
  *	FvwmErrorHandler - displays info on internal errors
  *
- ************************************************************************/
+ */
 int FvwmErrorHandler(Display *dpy, XErrorEvent *event)
 {
 #if 1
