@@ -1,3 +1,4 @@
+/* -*-c-*- */
 /* This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -13,9 +14,16 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#include "config.h"
+
 #include "types.h"
 #include "libs/fvwmsignal.h"
 #include "libs/ftime.h"
+#include "libs/FGettext.h"
+#include "libs/Bindings.h"
+#include "libs/charmap.h"
+#include "libs/wcontext.h"
+#include "libs/modifiers.h"
 #ifdef HAVE_GETPWUID
 #  include <pwd.h>
 #endif
@@ -23,8 +31,8 @@
 extern int fd[2];
 extern Window ref;
 
-void (*TabCom[26]) (int NbArg,long *TabArg);
-char *(*TabFunc[26]) (int *NbArg, long *TabArg);
+void (*TabCom[27]) (int NbArg,long *TabArg);
+char *(*TabFunc[27]) (int *NbArg, long *TabArg);
 int (*TabComp[7]) (char *arg1,char *arg2);
 
 extern Display *dpy;
@@ -48,9 +56,9 @@ char *BufCom;
 char Command[255]="None";
 time_t TimeCom=0;
 
-/*************************************************************/
-/* Utilities                                                 */
-/*************************************************************/
+/*
+ * Utilities
+ */
 void setFvwmUserDir(void)
 {
   char *home_dir;
@@ -78,9 +86,9 @@ void setFvwmUserDir(void)
   }
 }
 
-/*************************************************************/
-/* Ensemble de fonction de comparaison de deux entiers       */
-/*************************************************************/
+/*
+ * Ensemble de fonction de comparaison de deux entiers
+ */
 static int Inf(char *arg1,char *arg2)
 {
   int an1,an2;
@@ -129,9 +137,9 @@ static int Diff(char *arg1,char *arg2)
   return (strcmp(arg1,arg2)!=0);
 }
 
-/*****************************************************/
-/* Fonction qui retourne la valeur d'un argument     */
-/*****************************************************/
+/*
+ * Fonction qui retourne la valeur d'un argument
+ */
 static char *CalcArg (long *TabArg,int *Ix)
 {
   char *TmpStr;
@@ -160,11 +168,11 @@ static char *CalcArg (long *TabArg,int *Ix)
   return (TmpStr);
 }
 
-/*************************************************************/
-/* Ensemble des fonctions pour recuperer les prop d'un objet */
-/*************************************************************/
+/*
+ * Ensemble des fonctions pour recuperer les prop d'un objet
+ */
 
-/******* GetValue *******/
+/* GetValue */
 static char *FuncGetValue(int *NbArg, long *TabArg)
 {
   char *tmp;
@@ -179,7 +187,7 @@ static char *FuncGetValue(int *NbArg, long *TabArg)
   return tmp;
 }
 
-/******* GetMinValue *******/
+/* GetMinValue */
 static char *FuncGetMinValue(int *NbArg, long *TabArg)
 {
   char *tmp;
@@ -194,7 +202,7 @@ static char *FuncGetMinValue(int *NbArg, long *TabArg)
   return tmp;
 }
 
-/******* GetMaxValue *******/
+/* GetMaxValue */
 static char *FuncGetMaxValue(int *NbArg, long *TabArg)
 {
   char *tmp;
@@ -209,7 +217,7 @@ static char *FuncGetMaxValue(int *NbArg, long *TabArg)
   return tmp;
 }
 
-/******* GetFore *******/
+/* GetFore */
 static char *FuncGetFore(int *NbArg, long *TabArg)
 {
   char *tmp;
@@ -228,7 +236,7 @@ static char *FuncGetFore(int *NbArg, long *TabArg)
   return tmp;
 }
 
-/******* GetBack *******/
+/* GetBack */
 static char *FuncGetBack(int *NbArg, long *TabArg)
 {
   char *tmp;
@@ -247,7 +255,7 @@ static char *FuncGetBack(int *NbArg, long *TabArg)
   return tmp;
 }
 
-/******* GetHili *******/
+/* GetHili */
 static char *FuncGetHili(int *NbArg, long *TabArg)
 {
   char *tmp;
@@ -266,7 +274,7 @@ static char *FuncGetHili(int *NbArg, long *TabArg)
   return tmp;
 }
 
-/******* GetShad *******/
+/* GetShad */
 static char *FuncGetShad(int *NbArg, long *TabArg)
 {
   char *tmp;
@@ -285,7 +293,7 @@ static char *FuncGetShad(int *NbArg, long *TabArg)
   return tmp;
 }
 
-/******* Fonction qui retourne le titre d'un objet *******/
+/* Fonction qui retourne le titre d'un objet */
 static char *FuncGetTitle(int *NbArg, long *TabArg)
 {
   char *tmp;
@@ -307,7 +315,7 @@ static char *FuncGetTitle(int *NbArg, long *TabArg)
   return tmp;
 }
 
-/******* Fonction qui retourne la sortie d'une commande *******/
+/* Fonction qui retourne la sortie d'une commande */
 static char *FuncGetOutput(int *NbArg, long *TabArg)
 {
   char *cmndbuf;
@@ -395,7 +403,7 @@ static char *FuncGetOutput(int *NbArg, long *TabArg)
   return str;
 }
 
-/******* Convertion decimal vers hexadecimal *******/
+/* Convertion decimal vers hexadecimal */
 static char *FuncNumToHex(int *NbArg, long *TabArg)
 {
   char *str;
@@ -424,7 +432,7 @@ static char *FuncNumToHex(int *NbArg, long *TabArg)
   return str;
 }
 
-/******* Convertion hexadecimal vers decimal *******/
+/* Convertion hexadecimal vers decimal */
 static char *FuncHexToNum(int *NbArg, long *TabArg)
 {
   char *str,*str2;
@@ -442,7 +450,7 @@ static char *FuncHexToNum(int *NbArg, long *TabArg)
   return str2;
 }
 
-/******* + *******/
+/* + */
 static char *FuncAdd(int *NbArg, long *TabArg)
 {
   char *str;
@@ -461,7 +469,7 @@ static char *FuncAdd(int *NbArg, long *TabArg)
   return str;
 }
 
-/******* * *******/
+/* * */
 static char *FuncMult(int *NbArg, long *TabArg)
 {
   char *str;
@@ -480,7 +488,7 @@ static char *FuncMult(int *NbArg, long *TabArg)
   return str;
 }
 
-/******* / *******/
+/* / */
 static char *FuncDiv(int *NbArg, long *TabArg)
 {
   char *str;
@@ -499,7 +507,7 @@ static char *FuncDiv(int *NbArg, long *TabArg)
   return str;
 }
 
-/******* % *******/
+/* % */
 static char *RemainderOfDiv(int *NbArg, long *TabArg)
 {
 #ifndef HAVE_DIV
@@ -525,7 +533,7 @@ static char *RemainderOfDiv(int *NbArg, long *TabArg)
 }
 
 
-/******* StrCopy *******/
+/* StrCopy */
 static char *FuncStrCopy(int *NbArg, long *TabArg)
 {
   char *str,*strsrc;
@@ -560,7 +568,7 @@ static char *FuncStrCopy(int *NbArg, long *TabArg)
  return str;
 }
 
-/******* Lancement d'un script avec pipe *******/
+/* Lancement d'un script avec pipe */
 static char *LaunchScript (int *NbArg,long *TabArg)
 {
   char *arg,*execstr,*str,*scriptarg,*scriptname;
@@ -626,7 +634,7 @@ static char *LaunchScript (int *NbArg,long *TabArg)
   return str;
 }
 
-/******* GetScriptFather *******/
+/* GetScriptFather */
 static char *GetScriptFather (int *NbArg,long *TabArg)
 {
   char *str;
@@ -636,7 +644,7 @@ static char *GetScriptFather (int *NbArg,long *TabArg)
   return str;
 }
 
-/******* GetTime *******/
+/* GetTime */
 static char *GetTime (int *NbArg,long *TabArg)
 {
   char *str;
@@ -648,7 +656,7 @@ static char *GetTime (int *NbArg,long *TabArg)
   return str;
 }
 
-/******* GetScriptArg *******/
+/* GetScriptArg */
 static char *GetScriptArg (int *NbArg,long *TabArg)
 {
   char *str;
@@ -673,7 +681,7 @@ static char *GetScriptArg (int *NbArg,long *TabArg)
  return str;
 }
 
-/******* ReceivFromScript *******/
+/* ReceivFromScript */
 static char *ReceivFromScript (int *NbArg,long *TabArg)
 {
   char *arg,*msg;
@@ -744,7 +752,7 @@ static char *ReceivFromScript (int *NbArg,long *TabArg)
   return msg;
 }
 
-/******* GetPid *******/
+/* GetPid */
 static char *FuncGetPid(int *NbArg,long *TabArg)
 {
   char *str;
@@ -756,7 +764,7 @@ static char *FuncGetPid(int *NbArg,long *TabArg)
   return str;
 }
 
-/*******  SendMsgAndGet *******/
+/*  SendMsgAndGet */
 #define IN_FIFO_NBR_OF_TRY 200
 #define IN_FIFO_TIMEOUT    100000 /* usec: 0.1 sec (20 sec) */
 #define OUT_FIFO_NBR_OF_TRY  400
@@ -911,7 +919,7 @@ static char *FuncSendMsgAndGet(int *NbArg,long *TabArg)
   return str;
 }
 
-/******* Parse *******/
+/* Parse */
 static char *FuncParse(int *NbArg,long *TabArg)
 {
   char *string,*str;
@@ -960,7 +968,7 @@ static char *FuncParse(int *NbArg,long *TabArg)
   return str;
 }
 
-/******* GetLastString *******/
+/* GetLastString */
 static char *FuncGetLastString(int *NbArg,long *TabArg)
 {
   char *str;
@@ -976,11 +984,11 @@ static char *FuncGetLastString(int *NbArg,long *TabArg)
   return str;
 }
 
-/***********************************************/
-/* Ensemble des commandes possible pour un obj */
-/***********************************************/
+/*
+ * Ensemble des commandes possible pour un obj
+ */
 
-/******* Exec *******/
+/* Exec */
 static void Exec (int NbArg,long *TabArg)
 {
   unsigned long leng = 0;
@@ -1015,7 +1023,7 @@ static void Exec (int NbArg,long *TabArg)
   free(execstr);
 }
 
-/******* Hide *******/
+/* Hide */
 static void HideObj (int NbArg,long *TabArg)
 {
   char *arg[1];
@@ -1032,7 +1040,7 @@ static void HideObj (int NbArg,long *TabArg)
   free(arg[0]);
 }
 
-/******* Show *******/
+/* Show */
 static void ShowObj (int NbArg,long *TabArg)
 {
   char *arg[1];
@@ -1048,7 +1056,7 @@ static void ShowObj (int NbArg,long *TabArg)
   free(arg[0]);
 }
 
-/******* ChangeValue *******/
+/* ChangeValue */
 static void ChangeValue (int NbArg,long *TabArg)
 {
   int i = 0;
@@ -1067,7 +1075,7 @@ static void ChangeValue (int NbArg,long *TabArg)
   free(arg[1]);
 }
 
-/******* ChangeValueMax *******/
+/* ChangeValueMax */
 static void ChangeValueMax (int NbArg,long *TabArg)
 {
   int i=0;
@@ -1092,7 +1100,7 @@ static void ChangeValueMax (int NbArg,long *TabArg)
   free(arg[1]);
 }
 
-/******* ChangeValueMin *******/
+/* ChangeValueMin */
 static void ChangeValueMin (int NbArg,long *TabArg)
 {
   int i=0;
@@ -1117,7 +1125,7 @@ static void ChangeValueMin (int NbArg,long *TabArg)
   free(arg[1]);
 }
 
-/******* ChangePosition *******/
+/* ChangePosition */
 static void ChangePos (int NbArg,long *TabArg)
 {
   int i=0;
@@ -1143,7 +1151,7 @@ static void ChangePos (int NbArg,long *TabArg)
   free(arg[2]);
 }
 
-/******* ChangeFont *******/
+/* ChangeFont */
 static void ChangeFont (int NbArg,long *TabArg)
 {
   int i=0;
@@ -1179,7 +1187,7 @@ static void ChangeFont (int NbArg,long *TabArg)
   free(arg[1]);
 }
 
-/******* ChangeSize *******/
+/* ChangeSize */
 static void ChangeSize (int NbArg,long *TabArg)
 {
   int i=0;
@@ -1206,7 +1214,7 @@ static void ChangeSize (int NbArg,long *TabArg)
   free(arg[2]);
 }
 
-/******* ChangeTitle *******/
+/* ChangeTitle */
 static void ChangeTitle (int NbArg,long *TabArg)
 {
   int i=0;
@@ -1228,7 +1236,29 @@ static void ChangeTitle (int NbArg,long *TabArg)
   free(arg[1]);
 }
 
-/******* ChangeIcon *******/
+/* ChangeLocaleTitle */
+static void ChangeLocaleTitle (int NbArg,long *TabArg)
+{
+  int i=0;
+  char *arg[2];
+  int IdItem;
+
+  arg[0] = CalcArg(TabArg,&i);
+  i++;
+  arg[1] = CalcArg(TabArg,&i);
+  IdItem = TabIdObj[atoi(arg[0])];
+
+  if (tabxobj[IdItem]->title)
+    free(tabxobj[IdItem]->title);
+  tabxobj[IdItem]->title=safestrdup(FGettext(arg[1]));
+  if (tabxobj[IdItem]->TypeWidget != SwallowExec)
+    XClearWindow(dpy, tabxobj[IdItem]->win);
+  tabxobj[IdItem]->DrawObj(tabxobj[IdItem],NULL);
+  free(arg[0]);
+  free(arg[1]);
+}
+
+/* ChangeIcon */
 static void ChangeIcon (int NbArg,long *TabArg)
 {
 	int i=0;
@@ -1254,7 +1284,7 @@ static void ChangeIcon (int NbArg,long *TabArg)
 	free(arg[1]);
 }
 
-/******* ChangeForeColor *******/
+/* ChangeForeColor */
 static void ChangeForeColor (int NbArg,long *TabArg)
 {
   int i=0;
@@ -1294,7 +1324,7 @@ static void ChangeForeColor (int NbArg,long *TabArg)
 
 }
 
-/******* ChangeBackColor *******/
+/* ChangeBackColor */
 static void ChangeBackColor (int NbArg,long *TabArg)
 {
   int i=0;
@@ -1337,7 +1367,7 @@ static void ChangeBackColor (int NbArg,long *TabArg)
   free(arg[1]);
 }
 
-/******* ChangeMainColorset *******/
+/* ChangeMainColorset */
 static void ChangeMainColorset (int i)
 {
  /* Liberation de la couleur */
@@ -1357,7 +1387,7 @@ static void ChangeMainColorset (int i)
 		      &Colorset[i], Pdepth, x11base->gc, True);
 }
 
-/******* ChangeMainColorset *******/
+/* ChangeMainColorset */
 static void ChangeColorset (int NbArg,long *TabArg)
 {
   int i=0;
@@ -1402,7 +1432,7 @@ static void ChangeColorset (int NbArg,long *TabArg)
   free(arg[1]);
 }
 
-/******* SetVar *******/
+/* SetVar */
 static void SetVar (int NbArg,long *TabArg)
 {
   int i;
@@ -1422,7 +1452,7 @@ static void SetVar (int NbArg,long *TabArg)
   TabVVar[TabArg[0]] = str;
 }
 
-/******* SendSign *******/
+/* SendSign */
 static void SendSign (int NbArg,long *TabArg)
 {
   int i=0;
@@ -1440,7 +1470,7 @@ static void SendSign (int NbArg,long *TabArg)
   free(arg[1]);
 }
 
-/******* MyWarpPointer *******/
+/* MyWarpPointer */
 static void MyWarpPointer(int NbArg,long *TabArg)
 {
   int i=0;
@@ -1455,13 +1485,13 @@ static void MyWarpPointer(int NbArg,long *TabArg)
   free(arg);
 }
 
-/******* Quit *******/
+/* Quit */
 void Quit (int NbArg,long *TabArg)
 {
   exit(0);
 }
 
-/******* IfThen *******/
+/* IfThen */
 static void IfThen (int NbArg,long *TabArg)
 {
   char *arg[10];
@@ -1505,7 +1535,7 @@ static void IfThen (int NbArg,long *TabArg)
   free(arg[1]);
 }
 
-/******* Instruction boucle ********/
+/* Instruction boucle **/
 static void Loop (int NbArg,long *TabArg)
 {
   int IdVar;
@@ -1559,7 +1589,7 @@ static void Loop (int NbArg,long *TabArg)
  free(arg[1]);
 }
 
-/******** Instruction While ********/
+/** Instruction While **/
 static void While (int NbArg,long *TabArg)
 {
   char *arg[3],*str;
@@ -1585,7 +1615,7 @@ static void While (int NbArg,long *TabArg)
   }
 }
 
-/******* WriteToFile *******/
+/* WriteToFile */
 static void WriteToFile (int NbArg,long *TabArg)
 {
   int i=0;
@@ -1688,7 +1718,7 @@ static void WriteToFile (int NbArg,long *TabArg)
   free(arg[1]);
 }
 
-/******* SendToScript *******/
+/* SendToScript */
 static void SendToScript (int NbArg,long *TabArg)
 {
   char *tempstr,*Msg,*R;
@@ -1738,7 +1768,7 @@ static void SendToScript (int NbArg,long *TabArg)
   }
 }
 
-/******* Key *******/
+/* Key */
 static void Key (int NbArg,long *TabArg)
 {
   char *key_string,*in_modifier,*action,*str,*tmp,*widget,*sig;
@@ -1783,7 +1813,7 @@ static void Key (int NbArg,long *TabArg)
     error = 1;
   }
 
-  if (ParseModifiers(in_modifier, &modifier)) {
+  if (modifiers_string_to_modmask(in_modifier, &modifier)) {
     fprintf(stderr,"[%s][Key]: bad modifier: %s\n",
 	    ScriptName,in_modifier);
     error = 1;
@@ -1803,9 +1833,29 @@ static void Key (int NbArg,long *TabArg)
   free(in_modifier);
 }
 
-/****************************************************/
-/* Fonction d'initialisation de TabCom et TabFunc   */
-/****************************************************/
+/*
+ * GetText Support
+ */
+
+static char *FuncGettext(int *NbArg,long *TabArg)
+{
+	char *str;
+	char *string;
+
+	(*NbArg)++;
+	str = CalcArg(TabArg,NbArg);
+
+	string = FGettextCopy(str);
+	if (str && string != str)
+	{
+		free(str);
+	}
+	return (char *)string;
+}
+
+/*
+ * Fonction d'initialisation de TabCom et TabFunc
+ */
 void InitCom()
 {
   /* commande */
@@ -1833,6 +1883,7 @@ void InitCom()
   TabCom[23]=SendToScript;
   TabCom[24]=ChangeColorset;
   TabCom[25]=Key;
+  TabCom[26]=ChangeLocaleTitle;
 
   /* Fonction */
   TabFunc[1]=FuncGetValue;
@@ -1860,6 +1911,7 @@ void InitCom()
   TabFunc[23]=FuncSendMsgAndGet;
   TabFunc[24]=FuncParse;
   TabFunc[25]=FuncGetLastString;
+  TabFunc[26]=FuncGettext;
 
   /* Fonction de comparaison */
   TabComp[1]=Inf;
