@@ -1,3 +1,4 @@
+/* -*-c-*- */
 /* FvwmWinList Module for Fvwm.
  *
  *  Copyright 1994,  Mike Finger (mfinger@mermaid.micro.umn.edu or
@@ -27,21 +28,6 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-/* This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- */
-
 #include "config.h"
 
 #include <stdlib.h>
@@ -50,23 +36,16 @@
 
 #include "libs/fvwmlib.h"
 #include "libs/Colorset.h"
+#include "libs/Flocale.h"
 
 #include "FvwmWinList.h"
 #include "ButtonArray.h"
 #include "Mallocs.h"
+#include "libs/PictureGraphics.h"
+#include "libs/Rectangles.h"
 
-#ifdef I18gN_MB
-#ifdef __STDC__
-#define XTextWidth(x,y,z) XmbTextEscapement(x ## set,y,z)
-#else
-#define XTextWidth(x,y,z) XmbTextEscapement(x/**/set,y,z)
-#endif
-#endif
-
-extern XFontStruct *ButtonFont;
-#ifdef I18N_MB
-extern XFontSet ButtonFontset;
-#endif
+extern FlocaleFont *FButtonFont;
+extern FlocaleWinString *FwinString;
 extern Display *dpy;
 extern Window win;
 extern GC shadow[MAX_COLOUR_SETS],hilite[MAX_COLOUR_SETS];
@@ -74,21 +53,21 @@ extern GC graph[MAX_COLOUR_SETS],background[MAX_COLOUR_SETS];
 extern int colorset[MAX_COLOUR_SETS];
 extern Pixmap pixmap[MAX_COLOUR_SETS];
 extern int LeftJustify, TruncateLeft, ShowFocus;
+extern int win_width;
+extern int win_height;
 
 extern long CurrentDesk;
 extern int ShowCurrentDesk;
 extern int UseSkipList;
 
-/*************************************************************************
- *                                                                       *
- *  Button handling functions and procedures                             *
- *                                                                       *
- *************************************************************************/
+/*
+ * Button handling functions and procedures
+ */
 
-/* -------------------------------------------------------------------------
-   ButtonNew - Allocates and fills a new button structure
-   ------------------------------------------------------------------------- */
-Button *ButtonNew(char *title, Picture *p, int up)
+/*
+  ButtonNew - Allocates and fills a new button structure
+*/
+Button *ButtonNew(char *title, FvwmPicture *p, int up)
 {
   Button *new;
 
@@ -99,6 +78,7 @@ Button *ButtonNew(char *title, Picture *p, int up)
   {
     new->p.picture = p->picture;
     new->p.mask = p->mask;
+    new->p.alpha = p->alpha;
     new->p.width = p->width;
     new->p.height = p->height;
     new->p.depth = p->depth;
@@ -114,9 +94,9 @@ Button *ButtonNew(char *title, Picture *p, int up)
   return new;
 }
 
-/******************************************************************************
+/*
   InitArray - Initialize the arrary of buttons
-******************************************************************************/
+*/
 void InitArray(ButtonArray *array,int x,int y,int w,int h,int rw)
 {
   array->count=0;
@@ -128,9 +108,9 @@ void InitArray(ButtonArray *array,int x,int y,int w,int h,int rw)
   array->rw=rw;
 }
 
-/******************************************************************************
+/*
   UpdateArray - Update the array width
-******************************************************************************/
+*/
 void UpdateArray(ButtonArray *array, int w)
 {
   Button *temp;
@@ -143,9 +123,9 @@ void UpdateArray(ButtonArray *array, int w)
   }
 }
 
-/******************************************************************************
+/*
   Reorder buttons in window list order (see List.c:ReorderList()
-******************************************************************************/
+*/
 void ReorderButtons(ButtonArray *array, int ButNum, int FlipFocus)
 {
   Button *temp = array->head, *prev = NULL;
@@ -187,10 +167,10 @@ void ReorderButtons(ButtonArray *array, int ButNum, int FlipFocus)
       temp->needsupdate = True;
 }
 
-/******************************************************************************
+/*
   AddButton - Allocate space for and add the button to the bottom
-******************************************************************************/
-int AddButton(ButtonArray *array, char *title, Picture *p, int up)
+*/
+int AddButton(ButtonArray *array, char *title, FvwmPicture *p, int up)
 {
   Button *new;
 
@@ -208,7 +188,7 @@ int AddButton(ButtonArray *array, char *title, Picture *p, int up)
 
 /* in Taskbar this replaces below  ArrangeButtonArray (array);*/
 
-  new->tw=XTextWidth(ButtonFont,title,strlen(title));
+  new->tw=FlocaleTextWidth(FButtonFont,title,strlen(title));
   new->truncatewidth=0;
   new->next=NULL;
   new->needsupdate=1;
@@ -218,9 +198,9 @@ int AddButton(ButtonArray *array, char *title, Picture *p, int up)
   return (array->count-1);
 }
 
-/******************************************************************************
+/*
   UpdateButton - Change the name/state of a button
-******************************************************************************/
+*/
 int UpdateButton(ButtonArray *array, int butnum, char *title, int up)
 {
   Button *temp;
@@ -232,7 +212,7 @@ int UpdateButton(ButtonArray *array, int butnum, char *title, int up)
     {
       temp->title=(char *)saferealloc(temp->title,strlen(title)+1);
       strcpy(temp->title,title);
-      temp->tw=XTextWidth(ButtonFont,title,strlen(title));
+      temp->tw=FlocaleTextWidth(FButtonFont,title,strlen(title));
       temp->truncatewidth = 0;
     }
     if (up!=-1)
@@ -247,7 +227,7 @@ int UpdateButton(ButtonArray *array, int butnum, char *title, int up)
 /* -------------------------------------------------------------------------
    UpdateButtonPicture - Change the picture of a button
    ------------------------------------------------------------------------- */
-int UpdateButtonPicture(ButtonArray *array, int butnum, Picture *p)
+int UpdateButtonPicture(ButtonArray *array, int butnum, FvwmPicture *p)
 {
   Button *temp;
   temp=find_n(array,butnum);
@@ -256,6 +236,7 @@ int UpdateButtonPicture(ButtonArray *array, int butnum, Picture *p)
   {
     temp->p.picture = p->picture;
     temp->p.mask = p->mask;
+    temp->p.alpha = p->alpha;
     temp->p.width = p->width;
     temp->p.height = p->height;
     temp->p.depth = p->depth;
@@ -277,9 +258,9 @@ void UpdateButtonIconified(ButtonArray *array, int butnum, int iconified)
 }
 
 
-/******************************************************************************
+/*
   UpdateButtonSet - Change colour set of a button between odd and even
-******************************************************************************/
+*/
 int UpdateButtonSet(ButtonArray *array, int butnum, int set)
 {
   Button *btn;
@@ -296,9 +277,9 @@ int UpdateButtonSet(ButtonArray *array, int butnum, int set)
   return 1;
 }
 
-/******************************************************************************
+/*
   UpdateButtonDeskFlags - Change desk and flags of a button
-******************************************************************************/
+*/
 int UpdateButtonDeskFlags(ButtonArray *array, int butnum, long desk,
 			  int is_sticky, int skip)
 {
@@ -315,9 +296,9 @@ int UpdateButtonDeskFlags(ButtonArray *array, int butnum, long desk,
 }
 
 
-/******************************************************************************
+/*
   RemoveButton - Delete a button from the list
-******************************************************************************/
+*/
 void RemoveButton(ButtonArray *array, int butnum)
 {
   Button *temp, *to_die;
@@ -347,9 +328,9 @@ void RemoveButton(ButtonArray *array, int butnum)
     temp->needsupdate = 1;
 }
 
-/******************************************************************************
+/*
   find_n - Find the nth button in the list (Use internally)
-******************************************************************************/
+*/
 Button *find_n(ButtonArray *array, int n)
 {
   Button *temp;
@@ -360,9 +341,9 @@ Button *find_n(ButtonArray *array, int n)
   return temp;
 }
 
-/******************************************************************************
+/*
   FreeButton - Free space allocated to a button
-******************************************************************************/
+*/
 void FreeButton(Button *ptr)
 {
   if (ptr != NULL) {
@@ -371,9 +352,9 @@ void FreeButton(Button *ptr)
   }
 }
 
-/******************************************************************************
+/*
   FreeAllButtons - Free the whole array of buttons
-******************************************************************************/
+*/
 void FreeAllButtons(ButtonArray *array)
 {
 Button *temp,*temp2;
@@ -385,177 +366,289 @@ Button *temp,*temp2;
   }
 }
 
-/******************************************************************************
+/*
   DoButton - Draw the specified button.  (Used internally)
-******************************************************************************/
-void DoButton(Button *button, int x, int y, int w, int h, Bool clear_bg)
+*/
+void DoButton(
+	Button *button, int x, int y, int w, int h, Bool clear_bg, XEvent *evp)
 {
-  int up,Fontheight,newx,set,len;
-  GC topgc;
-  GC bottomgc;
-  char *string;
-  XGCValues gcv;
-  unsigned long gcm;
-  XFontStruct *font;
+	int up,Fontheight,newx,set,len;
+	GC topgc;
+	GC bottomgc;
+	char *string;
+	XGCValues gcv;
+	unsigned long gcm;
+	FlocaleFont *Ffont;
+	FvwmRenderAttributes fra;
+	XRectangle rect,inter;
+	Region t_region;
 
-  /* The margin we want between the relief/text/pixmaps */
+	/* The margin we want between the relief/text/pixmaps */
 #define INNER_MARGIN 2
 
-  h += 1;
-  up=button->up;
-  set=button->set;
-  topgc = up ? hilite[set] : shadow[set];
-  bottomgc = up ? shadow[set] : hilite[set];
-  font = ButtonFont;
-
-  gcm = GCFont;
-  gcv.font = font->fid;
-  XChangeGC(dpy, graph[set], gcm, &gcv);
-
-  Fontheight=ButtonFont->ascent+ButtonFont->descent;
-
-  /* handle transparency by clearing button, otherwise paint with background */
-  if (clear_bg) {
-    if (colorset[set] >= 0 && Colorset[colorset[set]].pixmap == ParentRelative)
-      XClearArea(dpy,win,x,y,w,h+1, False);
-    else
-      XFillRectangle(dpy,win,background[set],x,y,w,h+1);
-  }
-
-  if ((button->p.picture != 0)/* &&
-      (w + button->p.width + w3p + 3 > MIN_BUTTON_SIZE)*/) {
-    /* clip pixmap to fit inside button */
-    int height = min(button->p.height, h);
-    int offset = (button->p.height > h) ? 0 : ((h - button->p.height) >> 1);
-    gcm = GCClipMask|GCClipXOrigin|GCClipYOrigin;
-    gcv.clip_mask = button->p.mask;
-    gcv.clip_x_origin = x + 2 + button->reliefwidth;
-    gcv.clip_y_origin = y + offset;
-    XChangeGC(dpy, hilite[set], gcm, &gcv);
-    XCopyArea(dpy, button->p.picture, win, hilite[set], 0, 0,
-                   button->p.width, height,
-                   gcv.clip_x_origin, gcv.clip_y_origin);
-    gcm = GCClipMask;
-    gcv.clip_mask = None;
-    XChangeGC(dpy, hilite[set], gcm, &gcv);
-
-    newx = button->p.width+2*INNER_MARGIN;
-  }
-  else
-  {
-    if (LeftJustify)
-    {
-       newx = INNER_MARGIN;
-      if (!button->is_iconified)
-      {
-        static int icon_offset = -1;
-
-        if (icon_offset == -1)
-          icon_offset = XTextWidth(ButtonFont, "(", 1);
-        newx += icon_offset;
-      }
-    }
-    else
-      newx = max((w - button->tw) / 2 - button->reliefwidth, INNER_MARGIN);
-  }
-
-  /* check if the string needs to be truncated */
-  string = button->title;
-  len = strlen(string);
-
-  if (newx + button->tw + 2 * button->reliefwidth + INNER_MARGIN > w) {
-    if (button -> truncatewidth == w) {
-      /* truncated version already calculated, use it */
-      string = button->truncate_title;
-      len = strlen(string);
-    } else {
-      if (TruncateLeft) {
-	/* move the pointer up until the rest fits */
-	while (*string && (newx + XTextWidth(ButtonFont, string, strlen(string))
-			   + 2 * button->reliefwidth + INNER_MARGIN) > w) {
-	  string++;
-	  len--;
+	if (evp)
+	{
+		if (!frect_get_intersection(
+			x, y, w, h,
+			evp->xexpose.x, evp->xexpose.y,
+			evp->xexpose.width, evp->xexpose.height,
+			&rect))
+		{
+			return;
+		}
 	}
-	button->truncatewidth = w;
-	button->truncate_title = string;
-      } else {
-	while ((len > 1) && (newx + XTextWidth(ButtonFont, string, len)
-			   + 2 * button->reliefwidth + INNER_MARGIN) > w)
-	  len--;
-      }
-    }
-  }
-#ifdef I18N_MB
-  XmbDrawString(dpy,win,ButtonFontset,graph[set],x+newx+button->reliefwidth,
-              y+1+button->reliefwidth+ButtonFont->ascent,
-              string,len);
-#else
-  XDrawString(dpy,win,graph[set],x+newx+button->reliefwidth,
-              y+1+button->reliefwidth+ButtonFont->ascent,
-              string,len);
-#endif
+	else
+	{
+		rect.x = x;
+		rect.y = y;
+		rect.width = w;
+		rect.height = h;
+	}
+	up=button->up;
+	set=button->set;
+	topgc = up ? hilite[set] : shadow[set];
+	bottomgc = up ? shadow[set] : hilite[set];
+	Ffont = FButtonFont;
 
-  /* Draw relief last */
-  RelieveRectangle(dpy,win,x,y,w-1,h-1,topgc,bottomgc,button->reliefwidth);
+	if (Ffont->font != NULL)
+	{
+		gcm = GCFont;
+		gcv.font = Ffont->font->fid;
+		XChangeGC(dpy, graph[set], gcm, &gcv);
+	}
 
-  button->needsupdate=0;
+	Fontheight=FButtonFont->height;
+
+	if ((FftSupport && Ffont->fftf.fftfont != NULL) ||
+	    (button->p.picture != 0 && button->p.alpha != 0) ||
+	    (colorset[set] >= 0 &&
+	     Colorset[colorset[set]].icon_alpha_percent < 100))
+	{
+		clear_bg = True;
+	}
+
+	/* handle transparency by clearing button, otherwise paint with
+	 * background */
+	if (clear_bg)
+	{
+		if (CSET_IS_TRANSPARENT_PR_PURE(colorset[set]))
+		{
+			XClearArea(
+				dpy, win, rect.x, rect.y, rect.width,
+				rect.height, False);
+		}
+		else if (CSET_IS_TRANSPARENT_PR_TINT(colorset[set]))
+		{
+			SetRectangleBackground(
+				dpy, win, rect.x, rect.y, rect.width,
+				rect.height, &Colorset[colorset[set]], Pdepth,
+				background[set]);
+		}
+		else
+		{
+			XFillRectangle(
+				dpy, win, background[set],
+				rect.x, rect.y, rect.width, rect.height);
+		}
+	}
+
+	if (button->p.picture != 0)
+	{
+		/* clip pixmap to fit inside button */
+		int height = min(button->p.height, h);
+		int offset = (button->p.height > h) ?
+			0 : ((h - button->p.height) >> 1);
+
+		fra.mask = FRAM_DEST_IS_A_WINDOW;
+		if (colorset[set] >= 0)
+		{
+			fra.mask |= FRAM_HAVE_ICON_CSET;
+			fra.colorset = &Colorset[colorset[set]];
+		}
+		if (frect_get_intersection(
+			rect.x, rect.y, rect.width, rect.height,
+			x + 2 + button->reliefwidth, y+offset,
+			button->p.width, height, &inter))
+		{
+			PGraphicsRenderPicture(
+				dpy, win, &(button->p), &fra,
+				win, graph[set], None, None,
+				inter.x - (x + 2 + button->reliefwidth),
+				inter.y - (y+offset),
+				inter.width, inter.height,
+				inter.x, inter.y, inter.width, inter.height,
+				False);
+		}
+		newx = button->p.width+2*INNER_MARGIN;
+	}
+	else
+	{
+		if (LeftJustify)
+		{
+			newx = INNER_MARGIN;
+			if (!button->is_iconified)
+			{
+				static int icon_offset = -1;
+
+				if (icon_offset == -1)
+					icon_offset = FlocaleTextWidth(
+						FButtonFont, "(", 1);
+				newx += icon_offset;
+			}
+		}
+		else
+		{
+			newx = max(
+				(w - button->tw) / 2 - button->reliefwidth,
+				INNER_MARGIN);
+		}
+	}
+	/* check if the string needs to be truncated */
+	string = button->title;
+	len = strlen(string);
+
+	if (newx + button->tw + 2 * button->reliefwidth + INNER_MARGIN > w)
+	{
+		if (button -> truncatewidth == w)
+		{
+                        /* truncated version already calculated use it */
+			string = button->truncate_title;
+			len = strlen(string);
+		}
+		else
+		{
+			if (TruncateLeft)
+			{
+				/* move the ptr up until the rest fits */
+				while (*string &&
+				       (newx + FlocaleTextWidth(
+					       FButtonFont, string,
+					       strlen(string))
+					+ 2 * button->reliefwidth +
+					INNER_MARGIN) > w)
+				{
+					string++;
+					len--;
+				}
+				button->truncatewidth = w;
+				button->truncate_title = string;
+			}
+			else
+			{
+				while ((len > 1) &&
+				       (newx + FlocaleTextWidth(
+					       FButtonFont, string, len)
+					+ 2 * button->reliefwidth +
+					INNER_MARGIN) > w)
+				{
+					len--;
+				}
+			}
+		}
+	}
+	FwinString->str = string;
+	FwinString->len = len;
+	FwinString->x = x+newx+button->reliefwidth;
+	FwinString->y = y+1+button->reliefwidth+FButtonFont->ascent;
+	FwinString->gc = graph[set];
+	t_region = XCreateRegion();
+	XUnionRectWithRegion (&rect, t_region, t_region);
+	FwinString->flags.has_clip_region = True;
+	FwinString->clip_region = t_region;
+	FwinString->flags.has_colorset = False;
+	if (colorset[set] >= 0)
+	{
+		FwinString->colorset = &Colorset[colorset[set]] ;
+		FwinString->flags.has_colorset = True;
+	}
+	FlocaleDrawString(dpy, FButtonFont, FwinString, FWS_HAVE_LENGTH);
+	FwinString->flags.has_clip_region = False;
+	XDestroyRegion(t_region);
+
+	/* Draw relief last */
+	RelieveRectangle(dpy,win,x,y,w-1,h,topgc,bottomgc,button->reliefwidth);
+
+	button->needsupdate=0;
 }
 
-/******************************************************************************
+/*
   DrawButtonArray - Draw the whole array (all=1), or only those that need.
-******************************************************************************/
-void DrawButtonArray(ButtonArray *barray, Bool all, Bool clear_bg)
+*/
+void DrawButtonArray(
+	ButtonArray *barray, Bool all, Bool clear_bg, XEvent *evp)
 {
   Button *btn;
-  int i = 0;		/* buttons displayed */
+  int i = 0;            /* buttons displayed */
 
   for(btn = barray->head; btn != NULL; btn = btn->next)
     if (IsButtonVisible(btn))
     {
       if (all || btn->needsupdate)
-        DoButton(btn, barray->x, barray->y + (i * (barray->h + 1)),
-		 barray->w, barray->h, clear_bg);
+	DoButton(btn, barray->x, barray->y + (i * (barray->h + 1)),
+		 barray->w, barray->h, clear_bg, evp);
       i++;
     }
 }
 
-/******************************************************************************
-  ExposeAllButtons - Draw all button backgrounds intersectingwith the event
-******************************************************************************/
-void ExposeAllButtons(ButtonArray *barray, XEvent *eventp)
+/*
+  Mark all the buttons in the array as dirty
+*/
+void DirtyButtonArray(ButtonArray *barray)
 {
-  Button *btn;
-  int i, set;
-  XRectangle rect;
+	Button *btn;
 
-  rect.x = eventp->xexpose.x;
-  rect.y = eventp->xexpose.y;
-  rect.width = eventp->xexpose.width;
-  rect.height = eventp->xexpose.height;
+	for (btn = barray->head; btn != NULL; btn = btn->next)
+	{
+		btn->needsupdate = 1;
+	}
 
-  /* only fill the area exposed */
-  for (i = 0; i != MAX_COLOUR_SETS; i++)
-    XSetClipRectangles(dpy, background[i], 0, 0, &rect, 1, Unsorted);
-
-  i = 0;
-  for (btn = barray->head; btn != NULL; btn = btn->next)
-    if (IsButtonVisible(btn))
-    {
-      set = btn->set;
-      if (colorset[set] < 0 || Colorset[colorset[set]].pixmap != ParentRelative)
-	XFillRectangle(dpy, win, background[set],
-		       barray->x, barray->y + (i * (barray->h + 1)),
-		       barray->w, barray->h + 1);
-      i++;
-    }
-
-  /* clear the clipping */
-  for (i = 0; i != MAX_COLOUR_SETS; i++)
-    XSetClipMask(dpy, background[i], None);
+	return;
 }
-/******************************************************************************
+
+/*
+  DrawButtonArray - Draw the whole array (all=1), or only those that need.
+*/
+void DrawTransparentButtonArray(ButtonArray *barray)
+{
+	Button *btn;
+	int i = 0;            /* buttons displayed */
+
+	for (i = 0; i != MAX_COLOUR_SETS; i++)
+	{
+		int cset = colorset[i];
+
+		if (!CSET_IS_TRANSPARENT_ROOT(cset))
+		{
+			continue;
+		}
+		if (pixmap[i])
+			XFreePixmap(dpy, pixmap[i]);
+		pixmap[i] = CreateBackgroundPixmap(
+			dpy, win, win_width, win_height,
+			&Colorset[cset], Pdepth,
+			background[i], False);
+		XSetTile(dpy, background[i], pixmap[i]);
+	}
+
+	i = 0;
+	for(btn = barray->head; btn != NULL; btn = btn->next)
+	{
+		if (IsButtonVisible(btn))
+		{
+			if (CSET_IS_TRANSPARENT(colorset[btn->set]))
+			{
+				DoButton(
+					btn, barray->x,
+					barray->y + (i * (barray->h + 1)),
+					barray->w, barray->h, True, NULL);
+			}
+			i++;
+		}
+	}
+}
+/*
   SwitchButton - Alternate the state of a button
-******************************************************************************/
+*/
 void SwitchButton(ButtonArray *array, int butnum)
 {
   Button *btn;
@@ -565,7 +658,7 @@ void SwitchButton(ButtonArray *array, int butnum)
   {
     btn->up =!btn->up;
     btn->needsupdate=1;
-    DrawButtonArray(array, False, True);
+    DrawButtonArray(array, False, True, NULL);
   }
 }
 
@@ -583,42 +676,42 @@ void RadioButton(ButtonArray *array, int butnum, int butnumpressed)
     {
       if (ShowFocus && temp->up)
       {
-        temp->up = 0;
-        temp->needsupdate=1;
+	temp->up = 0;
+	temp->needsupdate=1;
       }
       if (!(temp->set & 2))
       {
-        temp->set |= 2;
-        temp->needsupdate=1;
+	temp->set |= 2;
+	temp->needsupdate=1;
       }
     }
     else if (i == butnumpressed)
     {
       if (temp->set & 2)
       {
-        temp->set &= 1;
-        temp->needsupdate = 1;
+	temp->set &= 1;
+	temp->needsupdate = 1;
       }
     }
     else
     {
       if (ShowFocus && !temp->up)
       {
-        temp->up = 1;
-        temp->needsupdate = 1;
+	temp->up = 1;
+	temp->needsupdate = 1;
       }
       if (temp->set & 2)
       {
-        temp->set &= 1;
-        temp->needsupdate = 1;
+	temp->set &= 1;
+	temp->needsupdate = 1;
       }
     }
   }
 }
 
-/******************************************************************************
+/*
   WhichButton - Based on x,y which button was pressed
-******************************************************************************/
+*/
 int WhichButton(ButtonArray *array,int x, int y)
 {
   int num;
@@ -639,9 +732,9 @@ int WhichButton(ButtonArray *array,int x, int y)
   return(num);
 }
 
-/******************************************************************************
+/*
   ButtonName - Return the name of the button
-******************************************************************************/
+*/
 char *ButtonName(ButtonArray *array, int butnum)
 {
   Button *temp;
@@ -650,9 +743,9 @@ char *ButtonName(ButtonArray *array, int butnum)
   return temp->title;
 }
 
-/******************************************************************************
+/*
   PrintButtons - Print the array of button names to the console. (Debugging)
-******************************************************************************/
+*/
 void PrintButtons(ButtonArray *array)
 {
   Button *temp;
@@ -662,22 +755,24 @@ void PrintButtons(ButtonArray *array)
     ConsoleMessage("   %s is %s\n",temp->title,(temp->up) ? "Up":"Down");
 }
 
-#ifdef MINI_ICONS
-/******************************************************************************
+/*
   ButtonPicture - Return the mini icon associated with the button
-******************************************************************************/
-Picture *ButtonPicture(ButtonArray *array, int butnum)
+*/
+FvwmPicture *ButtonPicture(ButtonArray *array, int butnum)
 {
   Button *temp;
 
+  if (!FMiniIconsSupported)
+  {
+    return NULL;
+  }
   temp=find_n(array,butnum);
   return &(temp->p);
 }
-#endif
 
-/******************************************************************************
+/*
   IsButtonVisible - Says if the button should be in winlist
-******************************************************************************/
+*/
 int IsButtonVisible(Button *btn)
 {
 
@@ -688,10 +783,10 @@ int IsButtonVisible(Button *btn)
     return 0;
 }
 
-/******************************************************************************
+/*
   IsButtonIndexVisible - Says if the button of index butnum should be
   in winlist
-******************************************************************************/
+*/
 int IsButtonIndexVisible(ButtonArray *array, int butnum)
 {
   Button *temp;
@@ -705,9 +800,9 @@ int IsButtonIndexVisible(ButtonArray *array, int butnum)
 }
 
 #if 0
-/******************************************************************************
+/*
   ButtonArrayMaxWidth - Calculate the width needed for the widest title
-******************************************************************************/
+*/
 int ButtonArrayMaxWidth(ButtonArray *array)
 {
 Button *temp;
